#! /usr/bin/env python

"""
Implementation of Elliptic-Curve Digital Signatures.

Classes and methods for elliptic-curve signatures:
private keys, public keys, signatures,
NIST prime-modulus curves with modulus lengths of
192, 224, 256, 384, and 521 bits.

Example:

  # (In real-life applications, you would probably want to
  # protect against defects in SystemRandom.)
  from random import SystemRandom
  randrange = SystemRandom().randrange

  # Generate a public/private key pair using the NIST Curve P-192:

  g = generator_192
  n = g.order()
  secret = randrange( 1, n )
  pubkey = Public_key( g, g * secret )
  privkey = Private_key( pubkey, secret )

  # Signing a hash value:

  hash = randrange( 1, n )
  signature = privkey.sign( hash, randrange( 1, n ) )

  # Verifying a signature for a hash value:

  if pubkey.verifies( hash, signature ):
    print_("Demo verification succeeded.")
  else:
    print_("*** Demo verification failed.")

  # Verification fails if the hash value is modified:

  if pubkey.verifies( hash-1, signature ):
    print_("**** Demo verification failed to reject tampered hash.")
  else:
    print_("Demo verification correctly rejected tampered hash.")

Version of 2009.05.16.

Revision history:
      2005.12.31 - Initial version.
      2008.11.25 - Substantial revisions introducing new classes.
      2009.05.16 - Warn against using random.randrange in real applications.
      2009.05.17 - Use random.SystemRandom by default.

Written in 2005 by Peter Pearson and placed in the public domain.
"""

from six import int2byte, b
from . import ellipticcurve
from . import numbertheory
from .util import bit_length


class RSZeroError(RuntimeError):
    pass


class InvalidPointError(RuntimeError):
    pass


class Signature(object):
  """ECDSA signature.
  """
  def __init__(self, r, s):
    self.r = r
    self.s = s

  def recover_public_keys(self, hash, generator):
    """Returns two public keys for which the signature is valid
    hash is signed hash
    generator is the used generator of the signature
    """
    curve = generator.curve()
    n = generator.order()
    r = self.r
    s = self.s
    e = hash
    x = r

    # Compute the curve point with x as x-coordinate
    alpha = (pow(x, 3, curve.p()) + (curve.a() * x) + curve.b()) % curve.p()
    beta = numbertheory.square_root_mod_prime(alpha, curve.p())
    y = beta if beta % 2 == 0 else curve.p() - beta

    # Compute the public key
    R1 = ellipticcurve.PointJacobi(curve, x, y, 1, n)
    Q1 = numbertheory.inverse_mod(r, n) * (s * R1 + (-e % n) * generator)
    Pk1 = Public_key(generator, Q1)

    # And the second solution
    R2 = ellipticcurve.PointJacobi(curve, x, -y, 1, n)
    Q2 = numbertheory.inverse_mod(r, n) * (s * R2 + (-e % n) * generator)
    Pk2 = Public_key(generator, Q2)

    return [Pk1, Pk2]


class Public_key(object):
  """Public key for ECDSA.
  """

  def __init__(self, generator, point, verify=True):
    """
    Low level ECDSA public key object.

    :param generator: the Point that generates the group (the base point)
    :param point: the Point that defines the public key
    :param bool verify: if True check if point is valid point on curve

    :raises InvalidPointError: if the point parameters are invalid or
        point does not lie on the curve
    """

    self.curve = generator.curve()
    self.generator = generator
    self.point = point
    n = generator.order()
    p = self.curve.p()
    if not (0 <= point.x() < p) or not (0 <= point.y() < p):
      raise InvalidPointError("The public point has x or y out of range.")
    if verify and not self.curve.contains_point(point.x(), point.y()):
        raise InvalidPointError("Point does not lie on the curve")
    if not n:
      raise InvalidPointError("Generator point must have order.")
    # for curve parameters with base point with cofactor 1, all points
    # that are on the curve are scalar multiples of the base point, so
    # verifying that is not necessary. See Section 3.2.2.1 of SEC 1 v2
    if verify and self.curve.cofactor() != 1 and \
            not n * point == ellipticcurve.INFINITY:
      raise InvalidPointError("Generator point order is bad.")

  def __eq__(self, other):
    if isinstance(other, Public_key):
      """Return True if the points are identical, False otherwise."""
      return self.curve == other.curve \
          and self.point == other.point
    return NotImplemented

  def verifies(self, hash, signature):
    """Verify that signature is a valid signature of hash.
    Return True if the signature is valid.
    """

    # From X9.62 J.3.1.

    G = self.generator
    n = G.order()
    r = signature.r
    s = signature.s
    if r < 1 or r > n - 1:
      return False
    if s < 1 or s > n - 1:
      return False
    c = numbertheory.inverse_mod(s, n)
    u1 = (hash * c) % n
    u2 = (r * c) % n
    if hasattr(G, "mul_add"):
        xy = G.mul_add(u1, self.point, u2)
    else:
        xy = u1 * G + u2 * self.point
    v = xy.x() % n
    return v == r


class Private_key(object):
  """Private key for ECDSA.
  """

  def __init__(self, public_key, secret_multiplier):
    """public_key is of class Public_key;
    secret_multiplier is a large integer.
    """

    self.public_key = public_key
    self.secret_multiplier = secret_multiplier

  def __eq__(self, other):
    if isinstance(other, Private_key):
      """Return True if the points are identical, False otherwise."""
      return self.public_key == other.public_key \
          and self.secret_multiplier == other.secret_multiplier
    return NotImplemented

  def sign(self, hash, random_k):
    """Return a signature for the provided hash, using the provided
    random nonce.  It is absolutely vital that random_k be an unpredictable
    number in the range [1, self.public_key.point.order()-1].  If
    an attacker can guess random_k, he can compute our private key from a
    single signature.  Also, if an attacker knows a few high-order
    bits (or a few low-order bits) of random_k, he can compute our private
    key from many signatures.  The generation of nonces with adequate
    cryptographic strength is very difficult and far beyond the scope
    of this comment.

    May raise RuntimeError, in which case retrying with a new
    random value k is in order.
    """

    G = self.public_key.generator
    n = G.order()
    k = random_k % n
    # Fix the bit-length of the random nonce,
    # so that it doesn't leak via timing.
    # This does not change that ks = k mod n
    ks = k + n
    kt = ks + n
    if bit_length(ks) == bit_length(n):
      p1 = kt * G
    else:
      p1 = ks * G
    r = p1.x() % n
    if r == 0:
      raise RSZeroError("amazingly unlucky random number r")
    s = (numbertheory.inverse_mod(k, n)
         * (hash + (self.secret_multiplier * r) % n)) % n
    if s == 0:
      raise RSZeroError("amazingly unlucky random number s")
    return Signature(r, s)


def int_to_string(x):
  """Convert integer x into a string of bytes, as per X9.62."""
  assert x >= 0
  if x == 0:
    return b('\0')
  result = []
  while x:
    ordinal = x & 0xFF
    result.append(int2byte(ordinal))
    x >>= 8

  result.reverse()
  return b('').join(result)


def string_to_int(s):
  """Convert a string of bytes into an integer, as per X9.62."""
  result = 0
  for c in s:
    if not isinstance(c, int):
      c = ord(c)
    result = 256 * result + c
  return result


def digest_integer(m):
  """Convert an integer into a string of bytes, compute
     its SHA-1 hash, and convert the result to an integer."""
  #
  # I don't expect this function to be used much. I wrote
  # it in order to be able to duplicate the examples
  # in ECDSAVS.
  #
  from hashlib import sha1
  return string_to_int(sha1(int_to_string(m)).digest())


def point_is_valid(generator, x, y):
  """Is (x,y) a valid public key based on the specified generator?"""

  # These are the tests specified in X9.62.

  n = generator.order()
  curve = generator.curve()
  p = curve.p()
  if not (0 <= x < p) or not (0 <= y < p):
    return False
  if not curve.contains_point(x, y):
    return False
  if curve.cofactor() != 1 and \
      not n * ellipticcurve.PointJacobi(curve, x, y, 1)\
          == ellipticcurve.INFINITY:
    return False
  return True


# NIST Curve P-192:
_p = 6277101735386680763835789423207666416083908700390324961279
_r = 6277101735386680763835789423176059013767194773182842284081
# s = 0x3045ae6fc8422f64ed579528d38120eae12196d5L
# c = 0x3099d2bbbfcb2538542dcd5fb078b6ef5f3d6fe2c745de65L
_b = 0x64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1
_Gx = 0x188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012
_Gy = 0x07192b95ffc8da78631011ed6b24cdd573f977a11e794811

curve_192 = ellipticcurve.CurveFp(_p, -3, _b, 1)
generator_192 = ellipticcurve.PointJacobi(
    curve_192, _Gx, _Gy, 1, _r, generator=True)


# NIST Curve P-224:
_p = 26959946667150639794667015087019630673557916260026308143510066298881
_r = 26959946667150639794667015087019625940457807714424391721682722368061
# s = 0xbd71344799d5c7fcdc45b59fa3b9ab8f6a948bc5L
# c = 0x5b056c7e11dd68f40469ee7f3c7a7d74f7d121116506d031218291fbL
_b = 0xb4050a850c04b3abf54132565044b0b7d7bfd8ba270b39432355ffb4
_Gx = 0xb70e0cbd6bb4bf7f321390b94a03c1d356c21122343280d6115c1d21
_Gy = 0xbd376388b5f723fb4c22dfe6cd4375a05a07476444d5819985007e34

curve_224 = ellipticcurve.CurveFp(_p, -3, _b, 1)
generator_224 = ellipticcurve.PointJacobi(
    curve_224, _Gx, _Gy, 1, _r, generator=True)

# NIST Curve P-256:
_p = 115792089210356248762697446949407573530086143415290314195533631308867097853951
_r = 115792089210356248762697446949407573529996955224135760342422259061068512044369
# s = 0xc49d360886e704936a6678e1139d26b7819f7e90L
# c = 0x7efba1662985be9403cb055c75d4f7e0ce8d84a9c5114abcaf3177680104fa0dL
_b = 0x5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b
_Gx = 0x6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296
_Gy = 0x4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5

curve_256 = ellipticcurve.CurveFp(_p, -3, _b, 1)
generator_256 = ellipticcurve.PointJacobi(
    curve_256, _Gx, _Gy, 1, _r, generator=True)

# NIST Curve P-384:
_p = 39402006196394479212279040100143613805079739270465446667948293404245721771496870329047266088258938001861606973112319
_r = 39402006196394479212279040100143613805079739270465446667946905279627659399113263569398956308152294913554433653942643
# s = 0xa335926aa319a27a1d00896a6773a4827acdac73L
# c = 0x79d1e655f868f02fff48dcdee14151ddb80643c1406d0ca10dfe6fc52009540a495e8042ea5f744f6e184667cc722483L
_b = 0xb3312fa7e23ee7e4988e056be3f82d19181d9c6efe8141120314088f5013875ac656398d8a2ed19d2a85c8edd3ec2aef
_Gx = 0xaa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a385502f25dbf55296c3a545e3872760ab7
_Gy = 0x3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c00a60b1ce1d7e819d7a431d7c90ea0e5f

curve_384 = ellipticcurve.CurveFp(_p, -3, _b, 1)
generator_384 = ellipticcurve.PointJacobi(
    curve_384, _Gx, _Gy, 1, _r, generator=True)

# NIST Curve P-521:
_p = 6864797660130609714981900799081393217269435300143305409394463459185543183397656052122559640661454554977296311391480858037121987999716643812574028291115057151
_r = 6864797660130609714981900799081393217269435300143305409394463459185543183397655394245057746333217197532963996371363321113864768612440380340372808892707005449
# s = 0xd09e8800291cb85396cc6717393284aaa0da64baL
# c = 0x0b48bfa5f420a34949539d2bdfc264eeeeb077688e44fbf0ad8f6d0edb37bd6b533281000518e19f1b9ffbe0fe9ed8a3c2200b8f875e523868c70c1e5bf55bad637L
_b = 0x051953eb9618e1c9a1f929a21a0b68540eea2da725b99b315f3b8b489918ef109e156193951ec7e937b1652c0bd3bb1bf073573df883d2c34f1ef451fd46b503f00
_Gx = 0xc6858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d3dbaa14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31c2e5bd66
_Gy = 0x11839296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e662c97ee72995ef42640c550b9013fad0761353c7086a272c24088be94769fd16650

curve_521 = ellipticcurve.CurveFp(_p, -3, _b, 1)
generator_521 = ellipticcurve.PointJacobi(
    curve_521, _Gx, _Gy, 1, _r, generator=True)

# Certicom secp256-k1
_a = 0x0000000000000000000000000000000000000000000000000000000000000000
_b = 0x0000000000000000000000000000000000000000000000000000000000000007
_p = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f
_Gx = 0x79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798
_Gy = 0x483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8
_r = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141

curve_secp256k1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_secp256k1 = ellipticcurve.PointJacobi(
    curve_secp256k1, _Gx, _Gy, 1, _r, generator=True)

# Brainpool P-160-r1
_a = 0x340E7BE2A280EB74E2BE61BADA745D97E8F7C300
_b = 0x1E589A8595423412134FAA2DBDEC95C8D8675E58
_p = 0xE95E4A5F737059DC60DFC7AD95B3D8139515620F
_Gx = 0xBED5AF16EA3F6A4F62938C4631EB5AF7BDBCDBC3
_Gy = 0x1667CB477A1A8EC338F94741669C976316DA6321
_q = 0xE95E4A5F737059DC60DF5991D45029409E60FC09

curve_brainpoolp160r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp160r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp160r1, _Gx, _Gy, 1, _q, generator=True)

# Brainpool P-192-r1
_a = 0x6A91174076B1E0E19C39C031FE8685C1CAE040E5C69A28EF
_b = 0x469A28EF7C28CCA3DC721D044F4496BCCA7EF4146FBF25C9
_p = 0xC302F41D932A36CDA7A3463093D18DB78FCE476DE1A86297
_Gx = 0xC0A0647EAAB6A48753B033C56CB0F0900A2F5C4853375FD6
_Gy = 0x14B690866ABD5BB88B5F4828C1490002E6773FA2FA299B8F
_q = 0xC302F41D932A36CDA7A3462F9E9E916B5BE8F1029AC4ACC1

curve_brainpoolp192r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp192r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp192r1, _Gx, _Gy, 1, _q, generator=True)

# Brainpool P-224-r1
_a = 0x68A5E62CA9CE6C1C299803A6C1530B514E182AD8B0042A59CAD29F43
_b = 0x2580F63CCFE44138870713B1A92369E33E2135D266DBB372386C400B
_p = 0xD7C134AA264366862A18302575D1D787B09F075797DA89F57EC8C0FF
_Gx = 0x0D9029AD2C7E5CF4340823B2A87DC68C9E4CE3174C1E6EFDEE12C07D
_Gy = 0x58AA56F772C0726F24C6B89E4ECDAC24354B9E99CAA3F6D3761402CD
_q = 0xD7C134AA264366862A18302575D0FB98D116BC4B6DDEBCA3A5A7939F

curve_brainpoolp224r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp224r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp224r1, _Gx, _Gy, 1, _q, generator=True)

# Brainpool P-256-r1
_a = 0x7D5A0975FC2C3057EEF67530417AFFE7FB8055C126DC5C6CE94A4B44F330B5D9
_b = 0x26DC5C6CE94A4B44F330B5D9BBD77CBF958416295CF7E1CE6BCCDC18FF8C07B6
_p = 0xA9FB57DBA1EEA9BC3E660A909D838D726E3BF623D52620282013481D1F6E5377
_Gx = 0x8BD2AEB9CB7E57CB2C4B482FFC81B7AFB9DE27E1E3BD23C23A4453BD9ACE3262
_Gy = 0x547EF835C3DAC4FD97F8461A14611DC9C27745132DED8E545C1D54C72F046997
_q = 0xA9FB57DBA1EEA9BC3E660A909D838D718C397AA3B561A6F7901E0E82974856A7

curve_brainpoolp256r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp256r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp256r1, _Gx, _Gy, 1, _q, generator=True)

# Brainpool P-320-r1
_a = 0x3EE30B568FBAB0F883CCEBD46D3F3BB8A2A73513F5EB79DA66190EB085FFA9F492F375A97D860EB4
_b = 0x520883949DFDBC42D3AD198640688A6FE13F41349554B49ACC31DCCD884539816F5EB4AC8FB1F1A6
_p = 0xD35E472036BC4FB7E13C785ED201E065F98FCFA6F6F40DEF4F92B9EC7893EC28FCD412B1F1B32E27
_Gx = 0x43BD7E9AFB53D8B85289BCC48EE5BFE6F20137D10A087EB6E7871E2A10A599C710AF8D0D39E20611
_Gy = 0x14FDD05545EC1CC8AB4093247F77275E0743FFED117182EAA9C77877AAAC6AC7D35245D1692E8EE1
_q = 0xD35E472036BC4FB7E13C785ED201E065F98FCFA5B68F12A32D482EC7EE8658E98691555B44C59311

curve_brainpoolp320r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp320r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp320r1, _Gx, _Gy, 1, _q, generator=True)

# Brainpool P-384-r1
_a = 0x7BC382C63D8C150C3C72080ACE05AFA0C2BEA28E4FB22787139165EFBA91F90F8AA5814A503AD4EB04A8C7DD22CE2826
_b = 0x04A8C7DD22CE28268B39B55416F0447C2FB77DE107DCD2A62E880EA53EEB62D57CB4390295DBC9943AB78696FA504C11
_p = 0x8CB91E82A3386D280F5D6F7E50E641DF152F7109ED5456B412B1DA197FB71123ACD3A729901D1A71874700133107EC53
_Gx = 0x1D1C64F068CF45FFA2A63A81B7C13F6B8847A3E77EF14FE3DB7FCAFE0CBD10E8E826E03436D646AAEF87B2E247D4AF1E
_Gy = 0x8ABE1D7520F9C2A45CB1EB8E95CFD55262B70B29FEEC5864E19C054FF99129280E4646217791811142820341263C5315
_q = 0x8CB91E82A3386D280F5D6F7E50E641DF152F7109ED5456B31F166E6CAC0425A7CF3AB6AF6B7FC3103B883202E9046565

curve_brainpoolp384r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp384r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp384r1, _Gx, _Gy, 1, _q, generator=True)

# Brainpool P-512-r1
_a = 0x7830A3318B603B89E2327145AC234CC594CBDD8D3DF91610A83441CAEA9863BC2DED5D5AA8253AA10A2EF1C98B9AC8B57F1117A72BF2C7B9E7C1AC4D77FC94CA
_b = 0x3DF91610A83441CAEA9863BC2DED5D5AA8253AA10A2EF1C98B9AC8B57F1117A72BF2C7B9E7C1AC4D77FC94CADC083E67984050B75EBAE5DD2809BD638016F723
_p = 0xAADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA703308717D4D9B009BC66842AECDA12AE6A380E62881FF2F2D82C68528AA6056583A48F3
_Gx = 0x81AEE4BDD82ED9645A21322E9C4C6A9385ED9F70B5D916C1B43B62EEF4D0098EFF3B1F78E2D0D48D50D1687B93B97D5F7C6D5047406A5E688B352209BCB9F822
_Gy = 0x7DDE385D566332ECC0EABFA9CF7822FDF209F70024A57B1AA000C55B881F8111B2DCDE494A5F485E5BCA4BD88A2763AED1CA2B2FA8F0540678CD1E0F3AD80892
_q = 0xAADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA70330870553E5C414CA92619418661197FAC10471DB1D381085DDADDB58796829CA90069

curve_brainpoolp512r1 = ellipticcurve.CurveFp(_p, _a, _b, 1)
generator_brainpoolp512r1 = ellipticcurve.PointJacobi(
    curve_brainpoolp512r1, _Gx, _Gy, 1, _q, generator=True)



try:
    import unittest2 as unittest
except ImportError:
    import unittest

try:
    buffer
except NameError:
    buffer = memoryview

import array
import six
import sys
import pytest
import hashlib

from .keys import VerifyingKey, SigningKey
from .der import unpem
from .util import sigencode_string, sigencode_der, sigencode_strings, \
    sigdecode_string, sigdecode_der, sigdecode_strings


class TestVerifyingKeyFromString(unittest.TestCase):
    """
    Verify that ecdsa.keys.VerifyingKey.from_string() can be used with
    bytes-like objects
    """

    @classmethod
    def setUpClass(cls):
        cls.key_bytes = (b'\x04L\xa2\x95\xdb\xc7Z\xd7\x1f\x93\nz\xcf\x97\xcf'
                       b'\xd7\xc2\xd9o\xfe8}X!\xae\xd4\xfah\xfa^\rpI\xba\xd1'
                       b'Y\xfb\x92xa\xebo+\x9cG\xfav\xca')
        cls.vk = VerifyingKey.from_string(cls.key_bytes)

    def test_bytes(self):
        self.assertIsNotNone(self.vk)
        self.assertIsInstance(self.vk, VerifyingKey)
        self.assertEqual(
            self.vk.pubkey.point.x(),
            105419898848891948935835657980914000059957975659675736097)
        self.assertEqual(
            self.vk.pubkey.point.y(),
            4286866841217412202667522375431381222214611213481632495306)

    def test_bytes_memoryview(self):
        vk = VerifyingKey.from_string(buffer(self.key_bytes))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytearray(self):
        vk = VerifyingKey.from_string(bytearray(self.key_bytes))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytesarray_memoryview(self):
        vk = VerifyingKey.from_string(buffer(bytearray(self.key_bytes)))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_array_array_of_bytes(self):
        arr = array.array('B', self.key_bytes)
        vk = VerifyingKey.from_string(arr)

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_array_array_of_bytes_memoryview(self):
        arr = array.array('B', self.key_bytes)
        vk = VerifyingKey.from_string(buffer(arr))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_array_array_of_ints(self):
        arr = array.array('I', self.key_bytes)
        vk = VerifyingKey.from_string(arr)

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_array_array_of_ints_memoryview(self):
        arr = array.array('I', self.key_bytes)
        vk = VerifyingKey.from_string(buffer(arr))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytes_uncompressed(self):
        vk = VerifyingKey.from_string(b'\x04' + self.key_bytes)

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytearray_uncompressed(self):
        vk = VerifyingKey.from_string(bytearray(b'\x04' + self.key_bytes))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytes_compressed(self):
        vk = VerifyingKey.from_string(b'\x02' + self.key_bytes[:24])

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytearray_compressed(self):
        vk = VerifyingKey.from_string(bytearray(b'\x02' + self.key_bytes[:24]))

        self.assertEqual(self.vk.to_string(), vk.to_string())


class TestVerifyingKeyFromDer(unittest.TestCase):
    """
    Verify that ecdsa.keys.VerifyingKey.from_der() can be used with
    bytes-like objects.
    """
    @classmethod
    def setUpClass(cls):
        prv_key_str = (
            "-----BEGIN EC PRIVATE KEY-----\n"
            "MF8CAQEEGF7IQgvW75JSqULpiQQ8op9WH6Uldw6xxaAKBggqhkjOPQMBAaE0AzIA\n"
            "BLiBd9CE7xf15FY5QIAoNg+fWbSk1yZOYtoGUdzkejWkxbRc9RWTQjqLVXucIJnz\n"
            "bA==\n"
            "-----END EC PRIVATE KEY-----\n")
        key_str = (
            "-----BEGIN PUBLIC KEY-----\n"
            "MEkwEwYHKoZIzj0CAQYIKoZIzj0DAQEDMgAEuIF30ITvF/XkVjlAgCg2D59ZtKTX\n"
            "Jk5i2gZR3OR6NaTFtFz1FZNCOotVe5wgmfNs\n"
            "-----END PUBLIC KEY-----\n")
        cls.key_pem = key_str

        cls.key_bytes = unpem(key_str)
        assert isinstance(cls.key_bytes, bytes)
        cls.vk = VerifyingKey.from_pem(key_str)
        cls.sk = SigningKey.from_pem(prv_key_str)

        key_str = (
            "-----BEGIN PUBLIC KEY-----\n"
            "MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAE4H3iRbG4TSrsSRb/gusPQB/4YcN8\n"
            "Poqzgjau4kfxBPyZimeRfuY/9g/wMmPuhGl4BUve51DsnKJFRr8psk0ieA==\n"
            "-----END PUBLIC KEY-----\n"
        )
        cls.vk2 = VerifyingKey.from_pem(key_str)

    def test_custom_hashfunc(self):
        vk = VerifyingKey.from_der(self.key_bytes, hashlib.sha256)

        self.assertIs(vk.default_hashfunc, hashlib.sha256)

    def test_from_pem_with_custom_hashfunc(self):
        vk = VerifyingKey.from_pem(self.key_pem, hashlib.sha256)

        self.assertIs(vk.default_hashfunc, hashlib.sha256)

    def test_bytes(self):
        vk = VerifyingKey.from_der(self.key_bytes)

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytes_memoryview(self):
        vk = VerifyingKey.from_der(buffer(self.key_bytes))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytearray(self):
        vk = VerifyingKey.from_der(bytearray(self.key_bytes))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_bytesarray_memoryview(self):
        vk = VerifyingKey.from_der(buffer(bytearray(self.key_bytes)))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_array_array_of_bytes(self):
        arr = array.array('B', self.key_bytes)
        vk = VerifyingKey.from_der(arr)

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_array_array_of_bytes_memoryview(self):
        arr = array.array('B', self.key_bytes)
        vk = VerifyingKey.from_der(buffer(arr))

        self.assertEqual(self.vk.to_string(), vk.to_string())

    def test_equality_on_verifying_keys(self):
        self.assertEqual(self.vk, self.sk.get_verifying_key())

    def test_inequality_on_verifying_keys(self):
        self.assertNotEqual(self.vk, self.vk2)

    def test_inequality_on_verifying_keys_not_implemented(self):
        self.assertNotEqual(self.vk, None)


class TestSigningKey(unittest.TestCase):
    """
    Verify that ecdsa.keys.SigningKey.from_der() can be used with
    bytes-like objects.
    """
    @classmethod
    def setUpClass(cls):
        prv_key_str = (
            "-----BEGIN EC PRIVATE KEY-----\n"
            "MF8CAQEEGF7IQgvW75JSqULpiQQ8op9WH6Uldw6xxaAKBggqhkjOPQMBAaE0AzIA\n"
            "BLiBd9CE7xf15FY5QIAoNg+fWbSk1yZOYtoGUdzkejWkxbRc9RWTQjqLVXucIJnz\n"
            "bA==\n"
            "-----END EC PRIVATE KEY-----\n")
        cls.sk1 = SigningKey.from_pem(prv_key_str) 
        
        prv_key_str = (
            "-----BEGIN EC PRIVATE KEY-----\n"
            "MHcCAQEEIKlL2EAm5NPPZuXwxRf4nXMk0A80y6UUbiQ17be/qFhRoAoGCCqGSM49\n"
            "AwEHoUQDQgAE4H3iRbG4TSrsSRb/gusPQB/4YcN8Poqzgjau4kfxBPyZimeRfuY/\n"
            "9g/wMmPuhGl4BUve51DsnKJFRr8psk0ieA==\n"
            "-----END EC PRIVATE KEY-----\n")
        cls.sk2 = SigningKey.from_pem(prv_key_str) 
        
    def test_equality_on_signing_keys(self):        
        sk = SigningKey.from_secret_exponent(self.sk1.privkey.secret_multiplier, self.sk1.curve)
        self.assertEqual(self.sk1, sk)
        
    def test_inequality_on_signing_keys(self):
        self.assertNotEqual(self.sk1, self.sk2)
        
    def test_inequality_on_signing_keys_not_implemented(self):
        self.assertNotEqual(self.sk1, None)

# test VerifyingKey.verify()
prv_key_str = (
    "-----BEGIN EC PRIVATE KEY-----\n"
    "MF8CAQEEGF7IQgvW75JSqULpiQQ8op9WH6Uldw6xxaAKBggqhkjOPQMBAaE0AzIA\n"
    "BLiBd9CE7xf15FY5QIAoNg+fWbSk1yZOYtoGUdzkejWkxbRc9RWTQjqLVXucIJnz\n"
    "bA==\n"
    "-----END EC PRIVATE KEY-----\n")
key_bytes = unpem(prv_key_str)
assert isinstance(key_bytes, bytes)
sk = SigningKey.from_der(key_bytes)
vk = sk.verifying_key

data = (b"some string for signing"
        b"contents don't really matter"
        b"but do include also some crazy values: "
        b"\x00\x01\t\r\n\x00\x00\x00\xff\xf0")
assert len(data) % 4 == 0
sha1 = hashlib.sha1()
sha1.update(data)
data_hash = sha1.digest()
assert isinstance(data_hash, bytes)
sig_raw = sk.sign(data, sigencode=sigencode_string)
assert isinstance(sig_raw, bytes)
sig_der = sk.sign(data, sigencode=sigencode_der)
assert isinstance(sig_der, bytes)
sig_strings = sk.sign(data, sigencode=sigencode_strings)
assert isinstance(sig_strings[0], bytes)

verifiers = []
for modifier, fun in [
    ("bytes", lambda x: x),
    ("bytes memoryview", lambda x: buffer(x)),
    ("bytearray", lambda x: bytearray(x)),
    ("bytearray memoryview", lambda x: buffer(bytearray(x))),
    ("array.array of bytes", lambda x: array.array('B', x)),
    ("array.array of bytes memoryview", lambda x: buffer(array.array('B', x))),
    ("array.array of ints", lambda x: array.array('I', x)),
    ("array.array of ints memoryview", lambda x: buffer(array.array('I', x)))
    ]:
    if "ints" in modifier:
        conv = lambda x: x
    else:
        conv = fun
    for sig_format, signature, decoder, mod_apply in [
        ("raw", sig_raw, sigdecode_string, lambda x: conv(x)),
        ("der", sig_der, sigdecode_der, lambda x: conv(x)),
        ("strings", sig_strings, sigdecode_strings, lambda x:
            tuple(conv(i) for i in x))
        ]:
        for method_name, vrf_mthd, vrf_data in [
            ("verify", vk.verify, data),
            ("verify_digest", vk.verify_digest, data_hash)
            ]:
            verifiers.append(pytest.param(
                signature, decoder, mod_apply, fun, vrf_mthd, vrf_data,
                id="{2}-{0}-{1}".format(modifier, sig_format, method_name)))

@pytest.mark.parametrize(
    "signature,decoder,mod_apply,fun,vrf_mthd,vrf_data",
    verifiers)
def test_VerifyingKey_verify(
        signature, decoder, mod_apply, fun, vrf_mthd, vrf_data):
    sig = mod_apply(signature)

    assert vrf_mthd(sig, fun(vrf_data), sigdecode=decoder)


# test SigningKey.from_string()
prv_key_bytes = (b'^\xc8B\x0b\xd6\xef\x92R\xa9B\xe9\x89\x04<\xa2'
                 b'\x9fV\x1f\xa5%w\x0e\xb1\xc5')
assert len(prv_key_bytes) == 24
converters = []
for modifier, convert in [
        ("bytes", lambda x: x),
        ("bytes memoryview", buffer),
        ("bytearray", bytearray),
        ("bytearray memoryview", lambda x: buffer(bytearray(x))),
        ("array.array of bytes", lambda x: array.array('B', x)),
        ("array.array of bytes memoryview",
         lambda x: buffer(array.array('B', x))),
        ("array.array of ints", lambda x: array.array('I', x)),
        ("array.array of ints memoryview",
         lambda x: buffer(array.array('I', x)))
        ]:
    converters.append(pytest.param(
        convert,
        id=modifier))

@pytest.mark.parametrize("convert", converters)
def test_SigningKey_from_string(convert):
    key = convert(prv_key_bytes)
    sk = SigningKey.from_string(key)

    assert sk.to_string() == prv_key_bytes


# test SigningKey.from_der()
prv_key_str = (
    "-----BEGIN EC PRIVATE KEY-----\n"
    "MF8CAQEEGF7IQgvW75JSqULpiQQ8op9WH6Uldw6xxaAKBggqhkjOPQMBAaE0AzIA\n"
    "BLiBd9CE7xf15FY5QIAoNg+fWbSk1yZOYtoGUdzkejWkxbRc9RWTQjqLVXucIJnz\n"
    "bA==\n"
    "-----END EC PRIVATE KEY-----\n")
key_bytes = unpem(prv_key_str)
assert isinstance(key_bytes, bytes)

# last two converters are for array.array of ints, those require input
# that's multiple of 4, which no curve we support produces
@pytest.mark.parametrize("convert", converters[:-2])
def test_SigningKey_from_der(convert):
    key = convert(key_bytes)
    sk = SigningKey.from_der(key)

    assert sk.to_string() == prv_key_bytes


# test SigningKey.sign_deterministic()
extra_entropy=b'\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11'

@pytest.mark.parametrize("convert", converters)
def test_SigningKey_sign_deterministic(convert):
    sig = sk.sign_deterministic(
        convert(data),
        extra_entropy=convert(extra_entropy))

    vk.verify(sig, data)


# test SigningKey.sign_digest_deterministic()
@pytest.mark.parametrize("convert", converters)
def test_SigningKey_sign_digest_deterministic(convert):
    sig = sk.sign_digest_deterministic(
        convert(data_hash),
        extra_entropy=convert(extra_entropy))

    vk.verify(sig, data)


@pytest.mark.parametrize("convert", converters)
def test_SigningKey_sign(convert):
    sig = sk.sign(convert(data))

    vk.verify(sig, data)


@pytest.mark.parametrize("convert", converters)
def test_SigningKey_sign_digest(convert):
    sig = sk.sign_digest(convert(data_hash))

    vk.verify(sig, data)



from .keys import SigningKey, VerifyingKey, BadSignatureError, BadDigestError,\
    MalformedPointError
from .curves import NIST192p, NIST224p, NIST256p, NIST384p, NIST521p,\
    SECP256k1, BRAINPOOLP160r1, BRAINPOOLP192r1, BRAINPOOLP224r1,\
    BRAINPOOLP256r1, BRAINPOOLP320r1, BRAINPOOLP384r1, BRAINPOOLP512r1
from .ecdh import ECDH, NoKeyError, NoCurveError, InvalidCurveError, \
    InvalidSharedSecretError
from .der import UnexpectedDER

# This code comes from http://github.com/warner/python-ecdsa
from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

__all__ = ["curves", "der", "ecdsa", "ellipticcurve", "keys", "numbertheory",
           "test_pyecdsa", "util", "six"]

_hush_pyflakes = [SigningKey, VerifyingKey, BadSignatureError, BadDigestError,
                  MalformedPointError, UnexpectedDER, InvalidCurveError,
                  NoKeyError, InvalidSharedSecretError, ECDH, NoCurveError,
                  NIST192p, NIST224p, NIST256p, NIST384p, NIST521p, SECP256k1,
                  BRAINPOOLP160r1, BRAINPOOLP192r1, BRAINPOOLP224r1,
                  BRAINPOOLP256r1, BRAINPOOLP320r1, BRAINPOOLP384r1,
                  BRAINPOOLP512r1]
del _hush_pyflakes




try:
    import unittest2 as unittest
except ImportError:
    import unittest

import hypothesis.strategies as st
from hypothesis import given, assume, settings, example

from .ellipticcurve import Point, PointJacobi, INFINITY
from .ecdsa import generator_256, curve_256, generator_224
from .numbertheory import inverse_mod

class TestJacobi(unittest.TestCase):
    def test___init__(self):
        curve = object()
        x = 2
        y = 3
        z = 1
        order = 4
        pj = PointJacobi(curve, x, y, z, order)

        self.assertEqual(pj.order(), order)
        self.assertIs(pj.curve(), curve)
        self.assertEqual(pj.x(), x)
        self.assertEqual(pj.y(), y)

    def test_add_with_different_curves(self):
        p_a = PointJacobi.from_affine(generator_256)
        p_b = PointJacobi.from_affine(generator_224)

        with self.assertRaises(ValueError):
            p_a + p_b

    def test_compare_different_curves(self):
        self.assertNotEqual(generator_256, generator_224)

    def test_equality_with_non_point(self):
        pj = PointJacobi.from_affine(generator_256)

        self.assertNotEqual(pj, "value")

    def test_conversion(self):
        pj = PointJacobi.from_affine(generator_256)
        pw = pj.to_affine()

        self.assertEqual(generator_256, pw)

    def test_single_double(self):
        pj = PointJacobi.from_affine(generator_256)
        pw = generator_256.double()

        pj = pj.double()

        self.assertEqual(pj.x(), pw.x())
        self.assertEqual(pj.y(), pw.y())

    def test_double_with_zero_point(self):
        pj = PointJacobi(curve_256, 0, 0, 1)

        pj = pj.double()

        self.assertIs(pj, INFINITY)

    def test_double_with_zero_equivalent_point(self):
        pj = PointJacobi(curve_256, 0, curve_256.p(), 1)

        pj = pj.double()

        self.assertIs(pj, INFINITY)

    def test_double_with_zero_equivalent_point_non_1_z(self):
        pj = PointJacobi(curve_256, 0, curve_256.p(), 2)

        pj = pj.double()

        self.assertIs(pj, INFINITY)

    def test_compare_with_affine_point(self):
        pj = PointJacobi.from_affine(generator_256)
        pa = pj.to_affine()

        self.assertEqual(pj, pa)
        self.assertEqual(pa, pj)

    def test_to_affine_with_zero_point(self):
        pj = PointJacobi(curve_256, 0, 0, 1)

        pa = pj.to_affine()

        self.assertIs(pa, INFINITY)

    def test_add_with_affine_point(self):
        pj = PointJacobi.from_affine(generator_256)
        pa = pj.to_affine()

        s = pj + pa

        self.assertEqual(s, pj.double())

    def test_radd_with_affine_point(self):
        pj = PointJacobi.from_affine(generator_256)
        pa = pj.to_affine()

        s = pa + pj

        self.assertEqual(s, pj.double())

    def test_add_with_infinity(self):
        pj = PointJacobi.from_affine(generator_256)

        s = pj + INFINITY

        self.assertEqual(s, pj)

    def test_add_zero_point_to_affine(self):
        pa = PointJacobi.from_affine(generator_256).to_affine()
        pj = PointJacobi(curve_256, 0, 0, 1)

        s = pj + pa

        self.assertIs(s, pa)

    def test_multiply_by_zero(self):
        pj = PointJacobi.from_affine(generator_256)

        pj = pj * 0

        self.assertIs(pj, INFINITY)

    def test_zero_point_multiply_by_one(self):
        pj = PointJacobi(curve_256, 0, 0, 1)

        pj = pj * 1

        self.assertIs(pj, INFINITY)

    def test_multiply_by_one(self):
        pj = PointJacobi.from_affine(generator_256)
        pw = generator_256 * 1

        pj = pj * 1

        self.assertEqual(pj.x(), pw.x())
        self.assertEqual(pj.y(), pw.y())

    def test_multiply_by_two(self):
        pj = PointJacobi.from_affine(generator_256)
        pw = generator_256 * 2

        pj = pj * 2

        self.assertEqual(pj.x(), pw.x())
        self.assertEqual(pj.y(), pw.y())

    def test_rmul_by_two(self):
        pj = PointJacobi.from_affine(generator_256)
        pw = generator_256 * 2

        pj = 2 * pj

        self.assertEqual(pj, pw)

    def test_compare_non_zero_with_infinity(self):
        pj = PointJacobi.from_affine(generator_256)

        self.assertNotEqual(pj, INFINITY)

    def test_compare_zero_point_with_infinity(self):
        pj = PointJacobi(curve_256, 0, 0, 1)

        self.assertEqual(pj, INFINITY)

    def test_compare_double_with_multiply(self):
        pj = PointJacobi.from_affine(generator_256)
        dbl = pj.double()
        mlpl = pj * 2

        self.assertEqual(dbl, mlpl)

    @settings(max_examples=10)
    @given(st.integers(min_value=0, max_value=int(generator_256.order())))
    def test_multiplications(self, mul):
        pj = PointJacobi.from_affine(generator_256)
        pw = pj.to_affine() * mul

        pj = pj * mul

        self.assertEqual((pj.x(), pj.y()), (pw.x(), pw.y()))
        self.assertEqual(pj, pw)

    @settings(max_examples=10)
    @given(st.integers(min_value=0, max_value=int(generator_256.order())))
    @example(0)
    @example(int(generator_256.order()))
    def test_precompute(self, mul):
        precomp = PointJacobi.from_affine(generator_256, True)
        pj = PointJacobi.from_affine(generator_256)

        a = precomp * mul
        b = pj * mul

        self.assertEqual(a, b)

    @settings(max_examples=10)
    @given(st.integers(min_value=1, max_value=int(generator_256.order())),
           st.integers(min_value=1, max_value=int(generator_256.order())))
    @example(3, 3)
    def test_add_scaled_points(self, a_mul, b_mul):
        j_g = PointJacobi.from_affine(generator_256)
        a = PointJacobi.from_affine(j_g * a_mul)
        b = PointJacobi.from_affine(j_g * b_mul)

        c = a + b

        self.assertEqual(c, j_g * (a_mul + b_mul))

    @settings(max_examples=10)
    @given(st.integers(min_value=1, max_value=int(generator_256.order())),
           st.integers(min_value=1, max_value=int(generator_256.order())),
           st.integers(min_value=1, max_value=int(curve_256.p()-1)))
    def test_add_one_scaled_point(self, a_mul, b_mul, new_z):
        j_g = PointJacobi.from_affine(generator_256)
        a = PointJacobi.from_affine(j_g * a_mul)
        b = PointJacobi.from_affine(j_g * b_mul)

        p = curve_256.p()

        assume(inverse_mod(new_z, p))

        new_zz = new_z * new_z % p

        b = PointJacobi(
            curve_256, b.x() * new_zz % p, b.y() * new_zz * new_z % p, new_z)

        c = a + b

        self.assertEqual(c, j_g * (a_mul + b_mul))

    @settings(max_examples=10)
    @given(st.integers(min_value=1, max_value=int(generator_256.order())),
           st.integers(min_value=1, max_value=int(generator_256.order())),
           st.integers(min_value=1, max_value=int(curve_256.p()-1)))
    @example(1, 1, 1)
    @example(3, 3, 3)
    @example(2, int(generator_256.order()-2), 1)
    @example(2, int(generator_256.order()-2), 3)
    def test_add_same_scale_points(self, a_mul, b_mul, new_z):
        j_g = PointJacobi.from_affine(generator_256)
        a = PointJacobi.from_affine(j_g * a_mul)
        b = PointJacobi.from_affine(j_g * b_mul)

        p = curve_256.p()

        assume(inverse_mod(new_z, p))

        new_zz = new_z * new_z % p

        a = PointJacobi(
            curve_256, a.x() * new_zz % p, a.y() * new_zz * new_z % p, new_z)
        b = PointJacobi(
            curve_256, b.x() * new_zz % p, b.y() * new_zz * new_z % p, new_z)

        c = a + b

        self.assertEqual(c, j_g * (a_mul + b_mul))

    @settings(max_examples=14)
    @given(st.integers(min_value=1, max_value=int(generator_256.order())),
           st.integers(min_value=1, max_value=int(generator_256.order())),
           st.lists(st.integers(min_value=1, max_value=int(curve_256.p()-1)),
                    min_size=2, max_size=2, unique=True))
    @example(2, 2, [2, 1])
    @example(2, 2, [2, 3])
    @example(2, int(generator_256.order()-2), [2, 3])
    @example(2, int(generator_256.order()-2), [2, 1])
    def test_add_different_scale_points(self, a_mul, b_mul, new_z):
        j_g = PointJacobi.from_affine(generator_256)
        a = PointJacobi.from_affine(j_g * a_mul)
        b = PointJacobi.from_affine(j_g * b_mul)

        p = curve_256.p()

        assume(inverse_mod(new_z[0], p))
        assume(inverse_mod(new_z[1], p))

        new_zz0 = new_z[0] * new_z[0] % p
        new_zz1 = new_z[1] * new_z[1] % p

        a = PointJacobi(
            curve_256,
            a.x() * new_zz0 % p,
            a.y() * new_zz0 * new_z[0] % p,
            new_z[0])
        b = PointJacobi(
            curve_256,
            b.x() * new_zz1 % p,
            b.y() * new_zz1 * new_z[1] % p,
            new_z[1])

        c = a + b

        self.assertEqual(c, j_g * (a_mul + b_mul))

    def test_add_point_3_times(self):
        j_g = PointJacobi.from_affine(generator_256)

        self.assertEqual(j_g * 3, j_g + j_g + j_g)

    def test_mul_add_inf(self):
        j_g = PointJacobi.from_affine(generator_256)

        self.assertEqual(j_g, j_g.mul_add(1, INFINITY, 1))

    def test_mul_add_same(self):
        j_g = PointJacobi.from_affine(generator_256)

        self.assertEqual(j_g * 2, j_g.mul_add(1, j_g, 1))

    def test_mul_add_precompute(self):
        j_g = PointJacobi.from_affine(generator_256, True)
        b = PointJacobi.from_affine(j_g * 255, True)

        self.assertEqual(j_g * 256, j_g + b)
        self.assertEqual(j_g * (5 + 255 * 7), j_g * 5 + b * 7)
        self.assertEqual(j_g * (5 + 255 * 7), j_g.mul_add(5, b, 7))

    def test_mul_add_precompute_large(self):
        j_g = PointJacobi.from_affine(generator_256, True)
        b = PointJacobi.from_affine(j_g * 255, True)

        self.assertEqual(j_g * 256, j_g + b)
        self.assertEqual(j_g * (0xff00 + 255 * 0xf0f0),
                         j_g * 0xff00 + b * 0xf0f0)
        self.assertEqual(j_g * (0xff00 + 255 * 0xf0f0),
                         j_g.mul_add(0xff00, b, 0xf0f0))

    def test_mul_add_to_mul(self):
        j_g = PointJacobi.from_affine(generator_256)

        a = j_g * 3
        b = j_g.mul_add(2, j_g, 1)

        self.assertEqual(a, b)

    def test_mul_add(self):
        j_g = PointJacobi.from_affine(generator_256)

        w_a = generator_256 * 255
        w_b = generator_256 * (0xa8*0xf0)
        j_b = j_g * 0xa8

        ret = j_g.mul_add(255, j_b, 0xf0)

        self.assertEqual(ret.to_affine(), w_a + w_b)

    def test_mul_add_large(self):
        j_g = PointJacobi.from_affine(generator_256)
        b = PointJacobi.from_affine(j_g * 255)

        self.assertEqual(j_g * 256, j_g + b)
        self.assertEqual(j_g * (0xff00 + 255 * 0xf0f0),
                         j_g * 0xff00 + b * 0xf0f0)
        self.assertEqual(j_g * (0xff00 + 255 * 0xf0f0),
                         j_g.mul_add(0xff00, b, 0xf0f0))



from __future__ import with_statement, division

import hashlib
try:
    from hashlib import algorithms_available
except ImportError:  # pragma: no cover
    algorithms_available = [
        "md5", "sha1", "sha224", "sha256", "sha384", "sha512"]
from functools import partial
import pytest
import sys
from six import binary_type
import hypothesis.strategies as st
from hypothesis import note, assume, given, settings, example

from .keys import SigningKey
from .keys import BadSignatureError
from .util import sigencode_der, sigencode_string
from .util import sigdecode_der, sigdecode_string
from .curves import curves, NIST256p
from .der import encode_integer, encode_bitstring, encode_octet_string, \
    encode_oid, encode_sequence, encode_constructed


example_data = b"some data to sign"
"""Since the data is hashed for processing, really any string will do."""


hash_and_size = [(name, hashlib.new(name).digest_size)
                 for name in algorithms_available]
"""Pairs of hash names and their output sizes.
Needed for pairing with curves as we don't support hashes
bigger than order sizes of curves."""


keys_and_sigs = []
"""Name of the curve+hash combination, VerifyingKey and DER signature."""


# for hypothesis strategy shrinking we want smallest curves and hashes first
for curve in sorted(curves, key=lambda x: x.baselen):
    for hash_alg in [name for name, size in
                     sorted(hash_and_size, key=lambda x: x[1])
                     if 0 < size <= curve.baselen]:
        sk = SigningKey.generate(
            curve,
            hashfunc=partial(hashlib.new, hash_alg))

        keys_and_sigs.append(
            ("{0} {1}".format(curve, hash_alg),
             sk.verifying_key,
             sk.sign(example_data, sigencode=sigencode_der)))


# first make sure that the signatures can be verified
@pytest.mark.parametrize(
    "verifying_key,signature",
    [pytest.param(vk, sig, id=name) for name, vk, sig in keys_and_sigs])
def test_signatures(verifying_key, signature):
    assert verifying_key.verify(signature, example_data,
                                sigdecode=sigdecode_der)


@st.composite
def st_fuzzed_sig(draw, keys_and_sigs):
    """
    Hypothesis strategy that generates pairs of VerifyingKey and malformed
    signatures created by fuzzing of a valid signature.
    """
    name, verifying_key, old_sig = draw(st.sampled_from(keys_and_sigs))
    note("Configuration: {0}".format(name))

    sig = bytearray(old_sig)

    # decide which bytes should be removed
    to_remove = draw(st.lists(
        st.integers(min_value=0, max_value=len(sig)-1),
        unique=True))
    to_remove.sort()
    for i in reversed(to_remove):
        del sig[i]
    note("Remove bytes: {0}".format(to_remove))

    # decide which bytes of the original signature should be changed
    if sig:  # pragma: no branch
        xors = draw(st.dictionaries(
            st.integers(min_value=0, max_value=len(sig)-1),
            st.integers(min_value=1, max_value=255)))
        for i, val in xors.items():
            sig[i] ^= val
        note("xors: {0}".format(xors))

    # decide where new data should be inserted
    insert_pos = draw(st.integers(min_value=0, max_value=len(sig)))
    # NIST521p signature is about 140 bytes long, test slightly longer
    insert_data = draw(st.binary(max_size=256))

    sig = sig[:insert_pos] + insert_data + sig[insert_pos:]
    note("Inserted at position {0} bytes: {1!r}"
         .format(insert_pos, insert_data))

    sig = bytes(sig)
    # make sure that there was performed at least one mutation on the data
    assume(to_remove or xors or insert_data)
    # and that the mutations didn't cancel each-other out
    assume(sig != old_sig)

    return verifying_key, sig


params = {}
# not supported in hypothesis 2.0.0
if sys.version_info >= (2, 7):  # pragma: no branch
    from hypothesis import HealthCheck
    # deadline=5s because NIST521p are slow to verify
    params["deadline"] = 5000
    params["suppress_health_check"] = [HealthCheck.data_too_large,
                                       HealthCheck.filter_too_much,
                                       HealthCheck.too_slow]

slow_params = dict(params)
slow_params["max_examples"] = 10


@settings(**params)
@given(st_fuzzed_sig(keys_and_sigs))
def test_fuzzed_der_signatures(args):
    verifying_key, sig = args

    with pytest.raises(BadSignatureError):
        verifying_key.verify(sig, example_data, sigdecode=sigdecode_der)


@st.composite
def st_random_der_ecdsa_sig_value(draw):
    """
    Hypothesis strategy for selecting random values and encoding them
    to ECDSA-Sig-Value object::

        ECDSA-Sig-Value ::= SEQUENCE {
            r INTEGER,
            s INTEGER
        }
    """
    name, verifying_key, _ = draw(st.sampled_from(keys_and_sigs))
    note("Configuration: {0}".format(name))
    order = int(verifying_key.curve.order)

    # the encode_integer doesn't suport negative numbers, would be nice
    # to generate them too, but we have coverage for remove_integer()
    # verifying that it doesn't accept them, so meh.
    # Test all numbers around the ones that can show up (around order)
    # way smaller and slightly bigger
    r = draw(st.integers(min_value=0, max_value=order << 4) |
             st.integers(min_value=order >> 2, max_value=order+1))
    s = draw(st.integers(min_value=0, max_value=order << 4) |
             st.integers(min_value=order >> 2, max_value=order+1))

    sig = encode_sequence(encode_integer(r), encode_integer(s))

    return verifying_key, sig


@settings(**slow_params)
@given(st_random_der_ecdsa_sig_value())
def test_random_der_ecdsa_sig_value(params):
    """
    Check if random values encoded in ECDSA-Sig-Value structure are rejected
    as signature.
    """
    verifying_key, sig = params

    with pytest.raises(BadSignatureError):
        verifying_key.verify(sig, example_data, sigdecode=sigdecode_der)


def st_der_integer(*args, **kwargs):
    """
    Hypothesis strategy that returns a random positive integer as DER
    INTEGER.
    Parameters are passed to hypothesis.strategy.integer.
    """
    if "min_value" not in kwargs:  # pragma: no branch
        kwargs["min_value"] = 0
    return st.builds(encode_integer, st.integers(*args, **kwargs))


@st.composite
def st_der_bit_string(draw, *args, **kwargs):
    """
    Hypothesis strategy that returns a random DER BIT STRING.
    Parameters are passed to hypothesis.strategy.binary.
    """
    data = draw(st.binary(*args, **kwargs))
    if data:
        unused = draw(st.integers(min_value=0, max_value=7))
        data = bytearray(data)
        data[-1] &= - (2**unused)
        data = bytes(data)
    else:
        unused = 0
    return encode_bitstring(data, unused)


def st_der_octet_string(*args, **kwargs):
    """
    Hypothesis strategy that returns a random DER OCTET STRING object.
    Parameters are passed to hypothesis.strategy.binary
    """
    return st.builds(encode_octet_string, st.binary(*args, **kwargs))


def st_der_null():
    """
    Hypothesis strategy that returns DER NULL object.
    """
    return st.just(b'\x05\x00')


@st.composite
def st_der_oid(draw):
    """
    Hypothesis strategy that returns DER OBJECT IDENTIFIER objects.
    """
    first = draw(st.integers(min_value=0, max_value=2))
    if first < 2:
        second = draw(st.integers(min_value=0, max_value=39))
    else:
        second = draw(st.integers(min_value=0, max_value=2**512))
    rest = draw(st.lists(st.integers(min_value=0, max_value=2**512),
                         max_size=50))
    return encode_oid(first, second, *rest)


def st_der():
    """
    Hypothesis strategy that returns random DER structures.

    A valid DER structure is any primitive object, an octet encoding
    of a valid DER structure, sequence of valid DER objects or a constructed
    encoding of any of the above.
    """
    return st.recursive(
        st.just(b'') | st_der_integer(max_value=2**4096) |
        st_der_bit_string(max_size=1024**2) |
        st_der_octet_string(max_size=1024**2) | st_der_null() | st_der_oid(),
        lambda children:
            st.builds(lambda x: encode_octet_string(x), st.one_of(children)) |
            st.builds(lambda x: encode_bitstring(x, 0), st.one_of(children)) |
            st.builds(lambda x: encode_sequence(*x),
                      st.lists(children, max_size=200)) |
            st.builds(lambda tag, x:
                      encode_constructed(tag, x),
                      st.integers(min_value=0, max_value=0x3f),
                      st.one_of(children)),
        max_leaves=40
        )


@settings(**params)
@given(st.sampled_from(keys_and_sigs), st_der())
def test_random_der_as_signature(params, der):
    """Check if random DER structures are rejected as signature"""
    name, verifying_key, _ = params

    with pytest.raises(BadSignatureError):
        verifying_key.verify(der, example_data, sigdecode=sigdecode_der)


@settings(**params)
@given(st.sampled_from(keys_and_sigs), st.binary(max_size=1024**2))
@example(
    keys_and_sigs[0],
    encode_sequence(encode_integer(0), encode_integer(0)))
@example(
    keys_and_sigs[0],
    encode_sequence(encode_integer(1), encode_integer(1)) + b'\x00')
@example(
    keys_and_sigs[0],
    encode_sequence(*[encode_integer(1)] * 3))
def test_random_bytes_as_signature(params, der):
    """Check if random bytes are rejected as signature"""
    name, verifying_key, _ = params

    with pytest.raises(BadSignatureError):
        verifying_key.verify(der, example_data, sigdecode=sigdecode_der)


keys_and_string_sigs = [
    (name, verifying_key,
     sigencode_string(*sigdecode_der(sig, verifying_key.curve.order),
                      order=verifying_key.curve.order))
    for name, verifying_key, sig in keys_and_sigs]
"""
Name of the curve+hash combination, VerifyingKey and signature as a
byte string.
"""


@settings(**params)
@given(st_fuzzed_sig(keys_and_string_sigs))
def test_fuzzed_string_signatures(params):
    verifying_key, sig = params

    with pytest.raises(BadSignatureError):
        verifying_key.verify(sig, example_data, sigdecode=sigdecode_string)



import operator
from six import print_
from functools import reduce
import operator
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import hypothesis.strategies as st
import pytest
from hypothesis import given, settings, example
try:
    from hypothesis import HealthCheck
    HC_PRESENT=True
except ImportError:  # pragma: no cover
    HC_PRESENT=False
from .numbertheory import (SquareRootError, factorization, gcd, lcm,
                           jacobi, inverse_mod,
                           is_prime, next_prime, smallprimes,
                           square_root_mod_prime)


BIGPRIMES = (999671,
             999683,
             999721,
             999727,
             999749,
             999763,
             999769,
             999773,
             999809,
             999853,
             999863,
             999883,
             999907,
             999917,
             999931,
             999953,
             999959,
             999961,
             999979,
             999983)


@pytest.mark.parametrize(
    "prime, next_p",
    [(p, q) for p, q in zip(BIGPRIMES[:-1], BIGPRIMES[1:])])
def test_next_prime(prime, next_p):
    assert next_prime(prime) == next_p


@pytest.mark.parametrize(
    "val",
    [-1, 0, 1])
def test_next_prime_with_nums_less_2(val):
    assert next_prime(val) == 2


@pytest.mark.parametrize("prime", smallprimes)
def test_square_root_mod_prime_for_small_primes(prime):
    squares = set()
    for num in range(0, 1 + prime // 2):
        sq = num * num % prime
        squares.add(sq)
        root = square_root_mod_prime(sq, prime)
        # tested for real with TestNumbertheory.test_square_root_mod_prime
        assert root * root % prime == sq

    for nonsquare in range(0, prime):
        if nonsquare in squares:
            continue
        with pytest.raises(SquareRootError):
            square_root_mod_prime(nonsquare, prime)


@st.composite
def st_two_nums_rel_prime(draw):
    # 521-bit is the biggest curve we operate on, use 1024 for a bit
    # of breathing space
    mod = draw(st.integers(min_value=2, max_value=2**1024))
    num = draw(st.integers(min_value=1, max_value=mod-1)
               .filter(lambda x: gcd(x, mod) == 1))
    return num, mod


@st.composite
def st_primes(draw, *args, **kwargs):
    if "min_value" not in kwargs:  # pragma: no branch
        kwargs["min_value"] = 1
    prime = draw(st.sampled_from(smallprimes) |
                 st.integers(*args, **kwargs)
                 .filter(is_prime))
    return prime


@st.composite
def st_num_square_prime(draw):
    prime = draw(st_primes(max_value=2**1024))
    num = draw(st.integers(min_value=0, max_value=1 + prime // 2))
    sq = num * num % prime
    return sq, prime


@st.composite
def st_comp_with_com_fac(draw):
    """
    Strategy that returns lists of numbers, all having a common factor.
    """
    primes = draw(st.lists(st_primes(max_value=2**512), min_size=1,
                           max_size=10))
    # select random prime(s) that will make the common factor of composites
    com_fac_primes = draw(st.lists(st.sampled_from(primes),
                                   min_size=1, max_size=20))
    com_fac = reduce(operator.mul, com_fac_primes, 1)

    # select at most 20 lists (returned numbers),
    # each having at most 30 primes (factors) including none (then the number
    # will be 1)
    comp_primes = draw(
        st.integers(min_value=1, max_value=20).
        flatmap(lambda n: st.lists(st.lists(st.sampled_from(primes),
                                            max_size=30),
                                   min_size=1, max_size=n)))

    return [reduce(operator.mul, nums, 1) * com_fac for nums in comp_primes]


@st.composite
def st_comp_no_com_fac(draw):
    """
    Strategy that returns lists of numbers that don't have a common factor.
    """
    primes = draw(st.lists(st_primes(max_value=2**512),
                           min_size=2, max_size=10, unique=True))
    # first select the primes that will create the uncommon factor
    # between returned numbers
    uncom_fac_primes = draw(st.lists(
        st.sampled_from(primes),
        min_size=1, max_size=len(primes)-1, unique=True))
    uncom_fac = reduce(operator.mul, uncom_fac_primes, 1)

    # then build composites from leftover primes
    leftover_primes = [i for i in primes if i not in uncom_fac_primes]

    assert leftover_primes
    assert uncom_fac_primes

    # select at most 20 lists, each having at most 30 primes
    # selected from the leftover_primes list
    number_primes = draw(
        st.integers(min_value=1, max_value=20).
        flatmap(lambda n: st.lists(st.lists(st.sampled_from(leftover_primes),
                                            max_size=30),
                                   min_size=1, max_size=n)))

    numbers = [reduce(operator.mul, nums, 1) for nums in number_primes]

    insert_at = draw(st.integers(min_value=0, max_value=len(numbers)))
    numbers.insert(insert_at, uncom_fac)
    return numbers


HYP_SETTINGS = {}
if HC_PRESENT:  # pragma: no branch
    HYP_SETTINGS['suppress_health_check']=[HealthCheck.filter_too_much,
                                           HealthCheck.too_slow]
    # the factorization() sometimes takes a long time to finish
    HYP_SETTINGS['deadline'] = 5000


HYP_SLOW_SETTINGS=dict(HYP_SETTINGS)
HYP_SLOW_SETTINGS["max_examples"] = 10


class TestNumbertheory(unittest.TestCase):
    def test_gcd(self):
        assert gcd(3 * 5 * 7, 3 * 5 * 11, 3 * 5 * 13) == 3 * 5
        assert gcd([3 * 5 * 7, 3 * 5 * 11, 3 * 5 * 13]) == 3 * 5
        assert gcd(3) == 3

    @unittest.skipUnless(HC_PRESENT,
                         "Hypothesis 2.0.0 can't be made tolerant of hard to "
                         "meet requirements (like `is_prime()`), the test "
                         "case times-out on it")
    @settings(**HYP_SLOW_SETTINGS)
    @given(st_comp_with_com_fac())
    def test_gcd_with_com_factor(self, numbers):
        n = gcd(numbers)
        assert 1 in numbers or n != 1
        for i in numbers:
            assert i % n == 0

    @unittest.skipUnless(HC_PRESENT,
                         "Hypothesis 2.0.0 can't be made tolerant of hard to "
                         "meet requirements (like `is_prime()`), the test "
                         "case times-out on it")
    @settings(**HYP_SLOW_SETTINGS)
    @given(st_comp_no_com_fac())
    def test_gcd_with_uncom_factor(self, numbers):
        n = gcd(numbers)
        assert n == 1

    @given(st.lists(st.integers(min_value=1, max_value=2**8192),
                    min_size=1, max_size=20))
    def test_gcd_with_random_numbers(self, numbers):
        n = gcd(numbers)
        for i in numbers:
            # check that at least it's a divider
            assert i % n == 0

    def test_lcm(self):
        assert lcm(3, 5 * 3, 7 * 3) == 3 * 5 * 7
        assert lcm([3, 5 * 3, 7 * 3]) == 3 * 5 * 7
        assert lcm(3) == 3

    @given(st.lists(st.integers(min_value=1, max_value=2**8192),
                    min_size=1, max_size=20))
    def test_lcm_with_random_numbers(self, numbers):
        n = lcm(numbers)
        for i in numbers:
            assert n % i == 0

    @unittest.skipUnless(HC_PRESENT,
                         "Hypothesis 2.0.0 can't be made tolerant of hard to "
                         "meet requirements (like `is_prime()`), the test "
                         "case times-out on it")
    @settings(**HYP_SETTINGS)
    @given(st_num_square_prime())
    def test_square_root_mod_prime(self, vals):
        square, prime = vals

        calc = square_root_mod_prime(square, prime)
        assert calc * calc % prime == square

    @settings(**HYP_SETTINGS)
    @given(st.integers(min_value=1, max_value=10**12))
    @example(265399 * 1526929)
    @example(373297 ** 2 * 553991)
    def test_factorization(self, num):
        factors = factorization(num)
        mult = 1
        for i in factors:
            mult *= i[0] ** i[1]
        assert mult == num

    @settings(**HYP_SETTINGS)
    @given(st.integers(min_value=3, max_value=1000).filter(lambda x: x % 2))
    def test_jacobi(self, mod):
        if is_prime(mod):
            squares = set()
            for root in range(1, mod):
                assert jacobi(root * root, mod) == 1
                squares.add(root * root % mod)
            for i in range(1, mod):
                if i not in squares:
                    assert jacobi(i, mod) == -1
        else:
            factors = factorization(mod)
            for a in range(1, mod):
                c = 1
                for i in factors:
                    c *= jacobi(a, i[0]) ** i[1]
                assert c == jacobi(a, mod)

    @given(st_two_nums_rel_prime())
    def test_inverse_mod(self, nums):
        num, mod = nums

        inv = inverse_mod(num, mod)

        assert 0 < inv < mod
        assert num * inv % mod == 1

    def test_inverse_mod_with_zero(self):
        assert 0 == inverse_mod(0, 11)



from __future__ import print_function
import sys
import hypothesis.strategies as st
from hypothesis import given, settings, note, example
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import pytest
from .ecdsa import Private_key, Public_key, Signature, \
    generator_192, digest_integer, ellipticcurve, point_is_valid, \
    generator_224, generator_256, generator_384, generator_521, \
    generator_secp256k1


HYP_SETTINGS = {}
# old hypothesis doesn't have the "deadline" setting
if sys.version_info > (2, 7):  # pragma: no branch
    # SEC521p is slow, allow long execution for it
    HYP_SETTINGS["deadline"] = 5000


class TestP192FromX9_62(unittest.TestCase):
    """Check test vectors from X9.62"""
    @classmethod
    def setUpClass(cls):
        cls.d = 651056770906015076056810763456358567190100156695615665659
        cls.Q = cls.d * generator_192
        cls.k = 6140507067065001063065065565667405560006161556565665656654
        cls.R = cls.k * generator_192

        cls.msg = 968236873715988614170569073515315707566766479517
        cls.pubk = Public_key(generator_192, generator_192 * cls.d)
        cls.privk = Private_key(cls.pubk, cls.d)
        cls.sig = cls.privk.sign(cls.msg, cls.k)

    def test_point_multiplication(self):
        assert self.Q.x() == 0x62B12D60690CDCF330BABAB6E69763B471F994DD702D16A5

    def test_point_multiplication_2(self):
        assert self.R.x() == 0x885052380FF147B734C330C43D39B2C4A89F29B0F749FEAD
        assert self.R.y() == 0x9CF9FA1CBEFEFB917747A3BB29C072B9289C2547884FD835

    def test_mult_and_addition(self):
        u1 = 2563697409189434185194736134579731015366492496392189760599
        u2 = 6266643813348617967186477710235785849136406323338782220568
        temp = u1 * generator_192 + u2 * self.Q
        assert temp.x() == 0x885052380FF147B734C330C43D39B2C4A89F29B0F749FEAD
        assert temp.y() == 0x9CF9FA1CBEFEFB917747A3BB29C072B9289C2547884FD835

    def test_signature(self):
        r, s = self.sig.r, self.sig.s
        assert r == 3342403536405981729393488334694600415596881826869351677613
        assert s == 5735822328888155254683894997897571951568553642892029982342

    def test_verification(self):
        assert self.pubk.verifies(self.msg, self.sig)

    def test_rejection(self):
        assert not self.pubk.verifies(self.msg - 1, self.sig)


class TestPublicKey(unittest.TestCase):
       
    def test_equality_public_keys(self):
        gen = generator_192
        x = 0xc58d61f88d905293bcd4cd0080bcb1b7f811f2ffa41979f6
        y = 0x8804dc7a7c4c7f8b5d437f5156f3312ca7d6de8a0e11867f        
        point = ellipticcurve.Point(gen.curve(), x, y)
        pub_key1 = Public_key(gen, point)
        pub_key2 = Public_key(gen, point)
        self.assertEqual(pub_key1, pub_key2)
        
    def test_inequality_public_key(self):
        gen = generator_192
        x1 = 0xc58d61f88d905293bcd4cd0080bcb1b7f811f2ffa41979f6
        y1 = 0x8804dc7a7c4c7f8b5d437f5156f3312ca7d6de8a0e11867f  
        point1 = ellipticcurve.Point(gen.curve(), x1, y1)
                
        x2 = 0x6a223d00bd22c52833409a163e057e5b5da1def2a197dd15
        y2 = 0x7b482604199367f1f303f9ef627f922f97023e90eae08abf
        point2 = ellipticcurve.Point(gen.curve(), x2, y2)
        
        pub_key1 = Public_key(gen, point1)
        pub_key2 = Public_key(gen, point2)
        self.assertNotEqual(pub_key1, pub_key2)
        
    def test_inequality_public_key_not_implemented(self):
        gen = generator_192
        x = 0xc58d61f88d905293bcd4cd0080bcb1b7f811f2ffa41979f6
        y = 0x8804dc7a7c4c7f8b5d437f5156f3312ca7d6de8a0e11867f        
        point = ellipticcurve.Point(gen.curve(), x, y)
        pub_key = Public_key(gen, point)
        self.assertNotEqual(pub_key, None)


class TestPrivateKey(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        gen = generator_192
        x = 0xc58d61f88d905293bcd4cd0080bcb1b7f811f2ffa41979f6
        y = 0x8804dc7a7c4c7f8b5d437f5156f3312ca7d6de8a0e11867f        
        point = ellipticcurve.Point(gen.curve(), x, y)
        cls.pub_key = Public_key(gen, point)
        
    def test_equality_private_keys(self):
        pr_key1 = Private_key(self.pub_key, 100)
        pr_key2 = Private_key(self.pub_key, 100)
        self.assertEqual(pr_key1, pr_key2)
        
    def test_inequality_private_keys(self):
        pr_key1 = Private_key(self.pub_key, 100)
        pr_key2 = Private_key(self.pub_key, 200)
        self.assertNotEqual(pr_key1, pr_key2)
        
    def test_inequality_private_keys_not_implemented(self):
        pr_key = Private_key(self.pub_key, 100)
        self.assertNotEqual(pr_key, None)
        

# Testing point validity, as per ECDSAVS.pdf B.2.2:
P192_POINTS = [
    (generator_192,
     0xcd6d0f029a023e9aaca429615b8f577abee685d8257cc83a,
     0x00019c410987680e9fb6c0b6ecc01d9a2647c8bae27721bacdfc,
     False),

    (generator_192,
     0x00017f2fce203639e9eaf9fb50b81fc32776b30e3b02af16c73b,
     0x95da95c5e72dd48e229d4748d4eee658a9a54111b23b2adb,
     False),

    (generator_192,
     0x4f77f8bc7fccbadd5760f4938746d5f253ee2168c1cf2792,
     0x000147156ff824d131629739817edb197717c41aab5c2a70f0f6,
     False),

    (generator_192,
     0xc58d61f88d905293bcd4cd0080bcb1b7f811f2ffa41979f6,
     0x8804dc7a7c4c7f8b5d437f5156f3312ca7d6de8a0e11867f,
     True),

    (generator_192,
     0xcdf56c1aa3d8afc53c521adf3ffb96734a6a630a4a5b5a70,
     0x97c1c44a5fb229007b5ec5d25f7413d170068ffd023caa4e,
     True),

    (generator_192,
     0x89009c0dc361c81e99280c8e91df578df88cdf4b0cdedced,
     0x27be44a529b7513e727251f128b34262a0fd4d8ec82377b9,
     True),

    (generator_192,
     0x6a223d00bd22c52833409a163e057e5b5da1def2a197dd15,
     0x7b482604199367f1f303f9ef627f922f97023e90eae08abf,
     True),

    (generator_192,
     0x6dccbde75c0948c98dab32ea0bc59fe125cf0fb1a3798eda,
     0x0001171a3e0fa60cf3096f4e116b556198de430e1fbd330c8835,
     False),

    (generator_192,
     0xd266b39e1f491fc4acbbbc7d098430931cfa66d55015af12,
     0x193782eb909e391a3148b7764e6b234aa94e48d30a16dbb2,
     False),

    (generator_192,
     0x9d6ddbcd439baa0c6b80a654091680e462a7d1d3f1ffeb43,
     0x6ad8efc4d133ccf167c44eb4691c80abffb9f82b932b8caa,
     False),

    (generator_192,
     0x146479d944e6bda87e5b35818aa666a4c998a71f4e95edbc,
     0xa86d6fe62bc8fbd88139693f842635f687f132255858e7f6,
     False),

    (generator_192,
     0xe594d4a598046f3598243f50fd2c7bd7d380edb055802253,
     0x509014c0c4d6b536e3ca750ec09066af39b4c8616a53a923,
     False)]


@pytest.mark.parametrize("generator,x,y,expected", P192_POINTS)
def test_point_validity(generator, x, y, expected):
    """
    `generator` defines the curve; is `(x, y)` a point on
    this curve? `expected` is True if the right answer is Yes.
    """
    assert point_is_valid(generator, x, y) == expected


# Trying signature-verification tests from ECDSAVS.pdf B.2.4:
CURVE_192_KATS = [
    (generator_192,
     int("0x84ce72aa8699df436059f052ac51b6398d2511e49631bcb7e71f89c499b9ee"
         "425dfbc13a5f6d408471b054f2655617cbbaf7937b7c80cd8865cf02c8487d30"
         "d2b0fbd8b2c4e102e16d828374bbc47b93852f212d5043c3ea720f086178ff79"
         "8cc4f63f787b9c2e419efa033e7644ea7936f54462dc21a6c4580725f7f0e7d1"
         "58", 16),
     0xd9dbfb332aa8e5ff091e8ce535857c37c73f6250ffb2e7ac,
     0x282102e364feded3ad15ddf968f88d8321aa268dd483ebc4,
     0x64dca58a20787c488d11d6dd96313f1b766f2d8efe122916,
     0x1ecba28141e84ab4ecad92f56720e2cc83eb3d22dec72479,
     True),

    (generator_192,
     int("0x94bb5bacd5f8ea765810024db87f4224ad71362a3c28284b2b9f39fab86db1"
         "2e8beb94aae899768229be8fdb6c4f12f28912bb604703a79ccff769c1607f5a"
         "91450f30ba0460d359d9126cbd6296be6d9c4bb96c0ee74cbb44197c207f6db3"
         "26ab6f5a659113a9034e54be7b041ced9dcf6458d7fb9cbfb2744d999f7dfd63"
         "f4", 16),
     0x3e53ef8d3112af3285c0e74842090712cd324832d4277ae7,
     0xcc75f8952d30aec2cbb719fc6aa9934590b5d0ff5a83adb7,
     0x8285261607283ba18f335026130bab31840dcfd9c3e555af,
     0x356d89e1b04541afc9704a45e9c535ce4a50929e33d7e06c,
     True),

    (generator_192,
     int("0xf6227a8eeb34afed1621dcc89a91d72ea212cb2f476839d9b4243c66877911"
         "b37b4ad6f4448792a7bbba76c63bdd63414b6facab7dc71c3396a73bd7ee14cd"
         "d41a659c61c99b779cecf07bc51ab391aa3252386242b9853ea7da67fd768d30"
         "3f1b9b513d401565b6f1eb722dfdb96b519fe4f9bd5de67ae131e64b40e78c42"
         "dd", 16),
     0x16335dbe95f8e8254a4e04575d736befb258b8657f773cb7,
     0x421b13379c59bc9dce38a1099ca79bbd06d647c7f6242336,
     0x4141bd5d64ea36c5b0bd21ef28c02da216ed9d04522b1e91,
     0x159a6aa852bcc579e821b7bb0994c0861fb08280c38daa09,
     False),

    (generator_192,
     int("0x16b5f93afd0d02246f662761ed8e0dd9504681ed02a253006eb36736b56309"
         "7ba39f81c8e1bce7a16c1339e345efabbc6baa3efb0612948ae51103382a8ee8"
         "bc448e3ef71e9f6f7a9676694831d7f5dd0db5446f179bcb737d4a526367a447"
         "bfe2c857521c7f40b6d7d7e01a180d92431fb0bbd29c04a0c420a57b3ed26ccd"
         "8a", 16),
     0xfd14cdf1607f5efb7b1793037b15bdf4baa6f7c16341ab0b,
     0x83fa0795cc6c4795b9016dac928fd6bac32f3229a96312c4,
     0x8dfdb832951e0167c5d762a473c0416c5c15bc1195667dc1,
     0x1720288a2dc13fa1ec78f763f8fe2ff7354a7e6fdde44520,
     False),

    (generator_192,
     int("0x08a2024b61b79d260e3bb43ef15659aec89e5b560199bc82cf7c65c77d3919"
         "2e03b9a895d766655105edd9188242b91fbde4167f7862d4ddd61e5d4ab55196"
         "683d4f13ceb90d87aea6e07eb50a874e33086c4a7cb0273a8e1c4408f4b846bc"
         "eae1ebaac1b2b2ea851a9b09de322efe34cebe601653efd6ddc876ce8c2f2072"
         "fb", 16),
     0x674f941dc1a1f8b763c9334d726172d527b90ca324db8828,
     0x65adfa32e8b236cb33a3e84cf59bfb9417ae7e8ede57a7ff,
     0x9508b9fdd7daf0d8126f9e2bc5a35e4c6d800b5b804d7796,
     0x36f2bf6b21b987c77b53bb801b3435a577e3d493744bfab0,
     False),

    (generator_192,
     int("0x1843aba74b0789d4ac6b0b8923848023a644a7b70afa23b1191829bbe4397c"
         "e15b629bf21a8838298653ed0c19222b95fa4f7390d1b4c844d96e645537e0aa"
         "e98afb5c0ac3bd0e4c37f8daaff25556c64e98c319c52687c904c4de7240a1cc"
         "55cd9756b7edaef184e6e23b385726e9ffcba8001b8f574987c1a3fedaaa83ca"
         "6d", 16),
     0x10ecca1aad7220b56a62008b35170bfd5e35885c4014a19f,
     0x04eb61984c6c12ade3bc47f3c629ece7aa0a033b9948d686,
     0x82bfa4e82c0dfe9274169b86694e76ce993fd83b5c60f325,
     0xa97685676c59a65dbde002fe9d613431fb183e8006d05633,
     False),

    (generator_192,
     int("0x5a478f4084ddd1a7fea038aa9732a822106385797d02311aeef4d0264f824f"
         "698df7a48cfb6b578cf3da416bc0799425bb491be5b5ecc37995b85b03420a98"
         "f2c4dc5c31a69a379e9e322fbe706bbcaf0f77175e05cbb4fa162e0da82010a2"
         "78461e3e974d137bc746d1880d6eb02aa95216014b37480d84b87f717bb13f76"
         "e1", 16),
     0x6636653cb5b894ca65c448277b29da3ad101c4c2300f7c04,
     0xfdf1cbb3fc3fd6a4f890b59e554544175fa77dbdbeb656c1,
     0xeac2ddecddfb79931a9c3d49c08de0645c783a24cb365e1c,
     0x3549fee3cfa7e5f93bc47d92d8ba100e881a2a93c22f8d50,
     False),

    (generator_192,
     int("0xc598774259a058fa65212ac57eaa4f52240e629ef4c310722088292d1d4af6"
         "c39b49ce06ba77e4247b20637174d0bd67c9723feb57b5ead232b47ea452d5d7"
         "a089f17c00b8b6767e434a5e16c231ba0efa718a340bf41d67ea2d295812ff1b"
         "9277daacb8bc27b50ea5e6443bcf95ef4e9f5468fe78485236313d53d1c68f6b"
         "a2", 16),
     0xa82bd718d01d354001148cd5f69b9ebf38ff6f21898f8aaa,
     0xe67ceede07fc2ebfafd62462a51e4b6c6b3d5b537b7caf3e,
     0x4d292486c620c3de20856e57d3bb72fcde4a73ad26376955,
     0xa85289591a6081d5728825520e62ff1c64f94235c04c7f95,
     False),

    (generator_192,
     int("0xca98ed9db081a07b7557f24ced6c7b9891269a95d2026747add9e9eb80638a"
         "961cf9c71a1b9f2c29744180bd4c3d3db60f2243c5c0b7cc8a8d40a3f9a7fc91"
         "0250f2187136ee6413ffc67f1a25e1c4c204fa9635312252ac0e0481d89b6d53"
         "808f0c496ba87631803f6c572c1f61fa049737fdacce4adff757afed4f05beb6"
         "58", 16),
     0x7d3b016b57758b160c4fca73d48df07ae3b6b30225126c2f,
     0x4af3790d9775742bde46f8da876711be1b65244b2b39e7ec,
     0x95f778f5f656511a5ab49a5d69ddd0929563c29cbc3a9e62,
     0x75c87fc358c251b4c83d2dd979faad496b539f9f2ee7a289,
     False),

    (generator_192,
     int("0x31dd9a54c8338bea06b87eca813d555ad1850fac9742ef0bbe40dad400e102"
         "88acc9c11ea7dac79eb16378ebea9490e09536099f1b993e2653cd50240014c9"
         "0a9c987f64545abc6a536b9bd2435eb5e911fdfde2f13be96ea36ad38df4ae9e"
         "a387b29cced599af777338af2794820c9cce43b51d2112380a35802ab7e396c9"
         "7a", 16),
     0x9362f28c4ef96453d8a2f849f21e881cd7566887da8beb4a,
     0xe64d26d8d74c48a024ae85d982ee74cd16046f4ee5333905,
     0xf3923476a296c88287e8de914b0b324ad5a963319a4fe73b,
     0xf0baeed7624ed00d15244d8ba2aede085517dbdec8ac65f5,
     True),

    (generator_192,
     int("0xb2b94e4432267c92f9fdb9dc6040c95ffa477652761290d3c7de312283f645"
         "0d89cc4aabe748554dfb6056b2d8e99c7aeaad9cdddebdee9dbc099839562d90"
         "64e68e7bb5f3a6bba0749ca9a538181fc785553a4000785d73cc207922f63e8c"
         "e1112768cb1de7b673aed83a1e4a74592f1268d8e2a4e9e63d414b5d442bd045"
         "6d", 16),
     0xcc6fc032a846aaac25533eb033522824f94e670fa997ecef,
     0xe25463ef77a029eccda8b294fd63dd694e38d223d30862f1,
     0x066b1d07f3a40e679b620eda7f550842a35c18b80c5ebe06,
     0xa0b0fb201e8f2df65e2c4508ef303bdc90d934016f16b2dc,
     False),

    (generator_192,
     int("0x4366fcadf10d30d086911de30143da6f579527036937007b337f7282460eae"
         "5678b15cccda853193ea5fc4bc0a6b9d7a31128f27e1214988592827520b214e"
         "ed5052f7775b750b0c6b15f145453ba3fee24a085d65287e10509eb5d5f602c4"
         "40341376b95c24e5c4727d4b859bfe1483d20538acdd92c7997fa9c614f0f839"
         "d7", 16),
     0x955c908fe900a996f7e2089bee2f6376830f76a19135e753,
     0xba0c42a91d3847de4a592a46dc3fdaf45a7cc709b90de520,
     0x1f58ad77fc04c782815a1405b0925e72095d906cbf52a668,
     0xf2e93758b3af75edf784f05a6761c9b9a6043c66b845b599,
     False),

    (generator_192,
     int("0x543f8af57d750e33aa8565e0cae92bfa7a1ff78833093421c2942cadf99866"
         "70a5ff3244c02a8225e790fbf30ea84c74720abf99cfd10d02d34377c3d3b412"
         "69bea763384f372bb786b5846f58932defa68023136cd571863b304886e95e52"
         "e7877f445b9364b3f06f3c28da12707673fecb4b8071de06b6e0a3c87da160ce"
         "f3", 16),
     0x31f7fa05576d78a949b24812d4383107a9a45bb5fccdd835,
     0x8dc0eb65994a90f02b5e19bd18b32d61150746c09107e76b,
     0xbe26d59e4e883dde7c286614a767b31e49ad88789d3a78ff,
     0x8762ca831c1ce42df77893c9b03119428e7a9b819b619068,
     False),

    (generator_192,
     int("0xd2e8454143ce281e609a9d748014dcebb9d0bc53adb02443a6aac2ffe6cb009f"
         "387c346ecb051791404f79e902ee333ad65e5c8cb38dc0d1d39a8dc90add502357"
         "2720e5b94b190d43dd0d7873397504c0c7aef2727e628eb6a74411f2e400c65670"
         "716cb4a815dc91cbbfeb7cfe8c929e93184c938af2c078584da045e8f8d1", 16),
     0x66aa8edbbdb5cf8e28ceb51b5bda891cae2df84819fe25c0,
     0x0c6bc2f69030a7ce58d4a00e3b3349844784a13b8936f8da,
     0xa4661e69b1734f4a71b788410a464b71e7ffe42334484f23,
     0x738421cf5e049159d69c57a915143e226cac8355e149afe9,
     False),

    (generator_192,
     int("0x6660717144040f3e2f95a4e25b08a7079c702a8b29babad5a19a87654bc5c5af"
         "a261512a11b998a4fb36b5d8fe8bd942792ff0324b108120de86d63f65855e5461"
         "184fc96a0a8ffd2ce6d5dfb0230cbbdd98f8543e361b3205f5da3d500fdc8bac6d"
         "b377d75ebef3cb8f4d1ff738071ad0938917889250b41dd1d98896ca06fb", 16),
     0xbcfacf45139b6f5f690a4c35a5fffa498794136a2353fc77,
     0x6f4a6c906316a6afc6d98fe1f0399d056f128fe0270b0f22,
     0x9db679a3dafe48f7ccad122933acfe9da0970b71c94c21c1,
     0x984c2db99827576c0a41a5da41e07d8cc768bc82f18c9da9,
     False)
    ]


@pytest.mark.parametrize("gen,msg,qx,qy,r,s,expected", CURVE_192_KATS)
def test_signature_validity(gen, msg, qx, qy, r, s, expected):
    """
    `msg` = message, `qx` and `qy` represent the base point on
    elliptic curve of `gen`, `r` and `s` are the signature, and
    `expected` is True iff the signature is expected to be valid."""
    pubk = Public_key(gen,
                      ellipticcurve.Point(gen.curve(), qx, qy))
    assert expected == pubk.verifies(digest_integer(msg), Signature(r, s))


@pytest.mark.parametrize("gen,msg,qx,qy,r,s,expected",
                         [x for x in CURVE_192_KATS if x[6]])
def test_pk_recovery(gen, msg, r, s, qx, qy, expected):
    del expected
    sign = Signature(r, s)
    pks = sign.recover_public_keys(digest_integer(msg), gen)

    assert pks

    # Test if the signature is valid for all found public keys
    for pk in pks:
        q = pk.point
        test_signature_validity(gen, msg, q.x(), q.y(), r, s, True)

    # Test if the original public key is in the set of found keys
    original_q = ellipticcurve.Point(gen.curve(), qx, qy)
    points = [pk.point for pk in pks]
    assert original_q in points


@st.composite
def st_random_gen_key_msg_nonce(draw):
    """Hypothesis strategy for test_sig_verify()."""
    name_gen = {
        "generator_192": generator_192,
        "generator_224": generator_224,
        "generator_256": generator_256,
        "generator_secp256k1": generator_secp256k1,
        "generator_384": generator_384,
        "generator_521": generator_521}
    name = draw(st.sampled_from(sorted(name_gen.keys())))
    note("Generator used: {0}".format(name))
    generator = name_gen[name]
    order = int(generator.order())

    key = draw(st.integers(min_value=1, max_value=order))
    msg = draw(st.integers(min_value=1, max_value=order))
    nonce = draw(st.integers(min_value=1, max_value=order+1) |
                 st.integers(min_value=order>>1, max_value=order))
    return generator, key, msg, nonce


SIG_VER_SETTINGS = dict(HYP_SETTINGS)
SIG_VER_SETTINGS["max_examples"] = 10
@settings(**SIG_VER_SETTINGS)
@example((generator_224, 4, 1, 1))
@given(st_random_gen_key_msg_nonce())
def test_sig_verify(args):
    """
    Check if signing and verification works for arbitrary messages and
    that signatures for other messages are rejected.
    """
    generator, sec_mult, msg, nonce = args

    pubkey = Public_key(generator, generator * sec_mult)
    privkey = Private_key(pubkey, sec_mult)

    signature = privkey.sign(msg, nonce)

    assert pubkey.verifies(msg, signature)

    assert not pubkey.verifies(msg - 1, signature)




# compatibility with Python 2.6, for that we need unittest2 package,
# which is not available on 3.3 or 3.4
import warnings
from binascii import hexlify
try:
    import unittest2 as unittest
except ImportError:
    import unittest
from six import b
import hypothesis.strategies as st
from hypothesis import given, example
import pytest
from ._compat import str_idx_as_int
from .curves import NIST256p, NIST224p
from .der import remove_integer, UnexpectedDER, read_length, encode_bitstring,\
        remove_bitstring, remove_object, encode_oid


class TestRemoveInteger(unittest.TestCase):
    # DER requires the integers to be 0-padded only if they would be
    # interpreted as negative, check if those errors are detected
    def test_non_minimal_encoding(self):
        with self.assertRaises(UnexpectedDER):
            remove_integer(b('\x02\x02\x00\x01'))

    def test_negative_with_high_bit_set(self):
        with self.assertRaises(UnexpectedDER):
            remove_integer(b('\x02\x01\x80'))

    def test_minimal_with_high_bit_set(self):
        val, rem = remove_integer(b('\x02\x02\x00\x80'))

        self.assertEqual(val, 0x80)
        self.assertFalse(rem)

    def test_two_zero_bytes_with_high_bit_set(self):
        with self.assertRaises(UnexpectedDER):
            remove_integer(b('\x02\x03\x00\x00\xff'))

    def test_zero_length_integer(self):
        with self.assertRaises(UnexpectedDER):
            remove_integer(b('\x02\x00'))

    def test_empty_string(self):
        with self.assertRaises(UnexpectedDER):
            remove_integer(b(''))

    def test_encoding_of_zero(self):
        val, rem = remove_integer(b('\x02\x01\x00'))

        self.assertEqual(val, 0)
        self.assertFalse(rem)

    def test_encoding_of_127(self):
        val, rem = remove_integer(b('\x02\x01\x7f'))

        self.assertEqual(val, 127)
        self.assertFalse(rem)

    def test_encoding_of_128(self):
        val, rem = remove_integer(b('\x02\x02\x00\x80'))

        self.assertEqual(val, 128)
        self.assertFalse(rem)


class TestReadLength(unittest.TestCase):
    # DER requires the lengths between 0 and 127 to be encoded using the short
    # form and lengths above that encoded with minimal number of bytes
    # necessary
    def test_zero_length(self):
        self.assertEqual((0, 1), read_length(b('\x00')))

    def test_two_byte_zero_length(self):
        with self.assertRaises(UnexpectedDER):
            read_length(b('\x81\x00'))

    def test_two_byte_small_length(self):
        with self.assertRaises(UnexpectedDER):
            read_length(b('\x81\x7f'))

    def test_long_form_with_zero_length(self):
        with self.assertRaises(UnexpectedDER):
            read_length(b('\x80'))

    def test_smallest_two_byte_length(self):
        self.assertEqual((128, 2), read_length(b('\x81\x80')))

    def test_zero_padded_length(self):
        with self.assertRaises(UnexpectedDER):
            read_length(b('\x82\x00\x80'))

    def test_two_three_byte_length(self):
        self.assertEqual((256, 3), read_length(b'\x82\x01\x00'))

    def test_empty_string(self):
        with self.assertRaises(UnexpectedDER):
            read_length(b(''))

    def test_length_overflow(self):
        with self.assertRaises(UnexpectedDER):
            read_length(b('\x83\x01\x00'))


class TestEncodeBitstring(unittest.TestCase):
    # DER requires BIT STRINGS to include a number of padding bits in the
    # encoded byte string, that padding must be between 0 and 7

    def test_old_call_convention(self):
        """This is the old way to use the function."""
        warnings.simplefilter('always')
        with pytest.warns(DeprecationWarning) as warns:
            der = encode_bitstring(b'\x00\xff')

        self.assertEqual(len(warns), 1)
        self.assertIn("unused= needs to be specified",
            warns[0].message.args[0])

        self.assertEqual(der, b'\x03\x02\x00\xff')

    def test_new_call_convention(self):
        """This is how it should be called now."""
        warnings.simplefilter('always')
        with pytest.warns(None) as warns:
            der = encode_bitstring(b'\xff', 0)

        # verify that new call convention doesn't raise Warnings
        self.assertEqual(len(warns), 0)

        self.assertEqual(der, b'\x03\x02\x00\xff')

    def test_implicit_unused_bits(self):
        """
        Writing bit string with already included the number of unused bits.
        """
        warnings.simplefilter('always')
        with pytest.warns(None) as warns:
            der = encode_bitstring(b'\x00\xff', None)

        # verify that new call convention doesn't raise Warnings
        self.assertEqual(len(warns), 0)

        self.assertEqual(der, b'\x03\x02\x00\xff')

    def test_explicit_unused_bits(self):
        der = encode_bitstring(b'\xff\xf0', 4)

        self.assertEqual(der, b'\x03\x03\x04\xff\xf0')

    def test_empty_string(self):
        self.assertEqual(encode_bitstring(b'', 0), b'\x03\x01\x00')

    def test_invalid_unused_count(self):
        with self.assertRaises(ValueError):
            encode_bitstring(b'\xff\x00', 8)

    def test_invalid_unused_with_empty_string(self):
        with self.assertRaises(ValueError):
            encode_bitstring(b'', 1)

    def test_non_zero_padding_bits(self):
        with self.assertRaises(ValueError):
            encode_bitstring(b'\xff', 2)


class TestRemoveBitstring(unittest.TestCase):
    def test_old_call_convention(self):
        """This is the old way to call the function."""
        warnings.simplefilter('always')
        with pytest.warns(DeprecationWarning) as warns:
            bits, rest = remove_bitstring(b'\x03\x02\x00\xff')

        self.assertEqual(len(warns), 1)
        self.assertIn("expect_unused= needs to be specified",
                      warns[0].message.args[0])

        self.assertEqual(bits, b'\x00\xff')
        self.assertEqual(rest, b'')

    def test_new_call_convention(self):
        warnings.simplefilter('always')
        with pytest.warns(None) as warns:
            bits, rest = remove_bitstring(b'\x03\x02\x00\xff', 0)

        self.assertEqual(len(warns), 0)

        self.assertEqual(bits, b'\xff')
        self.assertEqual(rest, b'')

    def test_implicit_unexpected_unused(self):
        warnings.simplefilter('always')
        with pytest.warns(None) as warns:
            bits, rest = remove_bitstring(b'\x03\x02\x00\xff', None)

        self.assertEqual(len(warns), 0)

        self.assertEqual(bits, (b'\xff', 0))
        self.assertEqual(rest, b'')

    def test_with_padding(self):
        ret, rest = remove_bitstring(b'\x03\x02\x04\xf0', None)

        self.assertEqual(ret, (b'\xf0', 4))
        self.assertEqual(rest, b'')

    def test_not_a_bitstring(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x02\x02\x00\xff', None)

    def test_empty_encoding(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x03\x00', None)

    def test_empty_string(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'', None)

    def test_no_length(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x03', None)

    def test_unexpected_number_of_unused_bits(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x03\x02\x00\xff', 1)

    def test_invalid_encoding_of_unused_bits(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x03\x03\x08\xff\x00', None)

    def test_invalid_encoding_of_empty_string(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x03\x01\x01', None)

    def test_invalid_padding_bits(self):
        with self.assertRaises(UnexpectedDER):
            remove_bitstring(b'\x03\x02\x01\xff', None)


class TestStrIdxAsInt(unittest.TestCase):
    def test_str(self):
        self.assertEqual(115, str_idx_as_int('str', 0))

    def test_bytes(self):
        self.assertEqual(115, str_idx_as_int(b'str', 0))

    def test_bytearray(self):
        self.assertEqual(115, str_idx_as_int(bytearray(b'str'), 0))


class TestEncodeOid(unittest.TestCase):
    def test_pub_key_oid(self):
        oid_ecPublicKey = encode_oid(1, 2, 840, 10045, 2, 1)
        self.assertEqual(hexlify(oid_ecPublicKey), b("06072a8648ce3d0201"))

    def test_nist224p_oid(self):
        self.assertEqual(hexlify(NIST224p.encoded_oid), b("06052b81040021"))

    def test_nist256p_oid(self):
        self.assertEqual(hexlify(NIST256p.encoded_oid),
                         b"06082a8648ce3d030107")

    def test_large_second_subid(self):
        # from X.690, section 8.19.5
        oid = encode_oid(2, 999, 3)
        self.assertEqual(oid, b'\x06\x03\x88\x37\x03')

    def test_with_two_subids(self):
        oid = encode_oid(2, 999)
        self.assertEqual(oid, b'\x06\x02\x88\x37')

    def test_zero_zero(self):
        oid = encode_oid(0, 0)
        self.assertEqual(oid, b'\x06\x01\x00')

    def test_with_wrong_types(self):
        with self.assertRaises((TypeError, AssertionError)):
            encode_oid(0, None)

    def test_with_small_first_large_second(self):
        with self.assertRaises(AssertionError):
            encode_oid(1, 40)

    def test_small_first_max_second(self):
        oid = encode_oid(1, 39)
        self.assertEqual(oid, b'\x06\x01\x4f')

    def test_with_invalid_first(self):
        with self.assertRaises(AssertionError):
            encode_oid(3, 39)


class TestRemoveObject(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.oid_ecPublicKey = encode_oid(1, 2, 840, 10045, 2, 1)

    def test_pub_key_oid(self):
        oid, rest = remove_object(self.oid_ecPublicKey)
        self.assertEqual(rest, b'')
        self.assertEqual(oid, (1, 2, 840, 10045, 2, 1))

    def test_with_extra_bytes(self):
        oid, rest = remove_object(self.oid_ecPublicKey + b'more')
        self.assertEqual(rest, b'more')
        self.assertEqual(oid, (1, 2, 840, 10045, 2, 1))

    def test_with_large_second_subid(self):
        # from X.690, section 8.19.5
        oid, rest = remove_object(b'\x06\x03\x88\x37\x03')
        self.assertEqual(rest, b'')
        self.assertEqual(oid, (2, 999, 3))

    def test_with_padded_first_subid(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06\x02\x80\x00')

    def test_with_padded_second_subid(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06\x04\x88\x37\x80\x01')

    def test_with_missing_last_byte_of_multi_byte(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06\x03\x88\x37\x83')

    def test_with_two_subids(self):
        oid, rest = remove_object(b'\x06\x02\x88\x37')
        self.assertEqual(rest, b'')
        self.assertEqual(oid, (2, 999))

    def test_zero_zero(self):
        oid, rest = remove_object(b'\x06\x01\x00')
        self.assertEqual(rest, b'')
        self.assertEqual(oid, (0, 0))

    def test_empty_string(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'')

    def test_missing_length(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06')

    def test_empty_oid(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06\x00')

    def test_empty_oid_overflow(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06\x01')

    def test_with_wrong_type(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x04\x02\x88\x37')

    def test_with_too_long_length(self):
        with self.assertRaises(UnexpectedDER):
            remove_object(b'\x06\x03\x88\x37')


@st.composite
def st_oid(draw, max_value=2**512, max_size=50):
    """
    Hypothesis strategy that returns valid OBJECT IDENTIFIERs as tuples

    :param max_value: maximum value of any single sub-identifier
    :param max_size: maximum length of the generated OID
    """
    first = draw(st.integers(min_value=0, max_value=2))
    if first < 2:
        second = draw(st.integers(min_value=0, max_value=39))
    else:
        second = draw(st.integers(min_value=0, max_value=max_value))
    rest = draw(st.lists(st.integers(min_value=0, max_value=max_value),
                         max_size=max_size))
    return (first, second) + tuple(rest)


@given(st_oid())
def test_oids(ids):
    encoded_oid = encode_oid(*ids)
    decoded_oid, rest = remove_object(encoded_oid)
    assert rest == b''
    assert decoded_oid == ids



from __future__ import division

from . import der, ecdsa
from .util import orderlen


# orderlen was defined in this module previously, so keep it in __all__,
# will need to mark it as deprecated later
__all__ = ["UnknownCurveError", "orderlen", "Curve", "NIST192p",
           "NIST224p", "NIST256p", "NIST384p", "NIST521p", "curves",
           "find_curve", "SECP256k1", "BRAINPOOLP160r1", "BRAINPOOLP192r1",
           "BRAINPOOLP224r1", "BRAINPOOLP256r1", "BRAINPOOLP320r1",
           "BRAINPOOLP384r1", "BRAINPOOLP512r1"]


class UnknownCurveError(Exception):
    pass


class Curve:
    def __init__(self, name, curve, generator, oid, openssl_name=None):
        self.name = name
        self.openssl_name = openssl_name  # maybe None
        self.curve = curve
        self.generator = generator
        self.order = generator.order()
        self.baselen = orderlen(self.order)
        self.verifying_key_length = 2*self.baselen
        self.signature_length = 2*self.baselen
        self.oid = oid
        self.encoded_oid = der.encode_oid(*oid)

    def __repr__(self):
        return self.name


# the NIST curves
NIST192p = Curve("NIST192p", ecdsa.curve_192,
                 ecdsa.generator_192,
                 (1, 2, 840, 10045, 3, 1, 1), "prime192v1")


NIST224p = Curve("NIST224p", ecdsa.curve_224,
                 ecdsa.generator_224,
                 (1, 3, 132, 0, 33), "secp224r1")


NIST256p = Curve("NIST256p", ecdsa.curve_256,
                 ecdsa.generator_256,
                 (1, 2, 840, 10045, 3, 1, 7), "prime256v1")


NIST384p = Curve("NIST384p", ecdsa.curve_384,
                 ecdsa.generator_384,
                 (1, 3, 132, 0, 34), "secp384r1")


NIST521p = Curve("NIST521p", ecdsa.curve_521,
                 ecdsa.generator_521,
                 (1, 3, 132, 0, 35), "secp521r1")


SECP256k1 = Curve("SECP256k1", ecdsa.curve_secp256k1,
                  ecdsa.generator_secp256k1,
                  (1, 3, 132, 0, 10), "secp256k1")


BRAINPOOLP160r1 = Curve("BRAINPOOLP160r1",
                        ecdsa.curve_brainpoolp160r1,
                        ecdsa.generator_brainpoolp160r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 1),
                        "brainpoolP160r1")


BRAINPOOLP192r1 = Curve("BRAINPOOLP192r1",
                        ecdsa.curve_brainpoolp192r1,
                        ecdsa.generator_brainpoolp192r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 3),
                        "brainpoolP192r1")


BRAINPOOLP224r1 = Curve("BRAINPOOLP224r1",
                        ecdsa.curve_brainpoolp224r1,
                        ecdsa.generator_brainpoolp224r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 5),
                        "brainpoolP224r1")


BRAINPOOLP256r1 = Curve("BRAINPOOLP256r1",
                        ecdsa.curve_brainpoolp256r1,
                        ecdsa.generator_brainpoolp256r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 7),
                        "brainpoolP256r1")


BRAINPOOLP320r1 = Curve("BRAINPOOLP320r1",
                        ecdsa.curve_brainpoolp320r1,
                        ecdsa.generator_brainpoolp320r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 9),
                        "brainpoolP320r1")


BRAINPOOLP384r1 = Curve("BRAINPOOLP384r1",
                        ecdsa.curve_brainpoolp384r1,
                        ecdsa.generator_brainpoolp384r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 11),
                        "brainpoolP384r1")


BRAINPOOLP512r1 = Curve("BRAINPOOLP512r1",
                        ecdsa.curve_brainpoolp512r1,
                        ecdsa.generator_brainpoolp512r1,
                        (1, 3, 36, 3, 3, 2, 8, 1, 1, 13),
                        "brainpoolP512r1")


curves = [NIST192p, NIST224p, NIST256p, NIST384p, NIST521p, SECP256k1,
          BRAINPOOLP160r1, BRAINPOOLP192r1, BRAINPOOLP224r1, BRAINPOOLP256r1,
          BRAINPOOLP320r1, BRAINPOOLP384r1, BRAINPOOLP512r1]


def find_curve(oid_curve):
    for c in curves:
        if c.oid == oid_curve:
            return c
    raise UnknownCurveError("I don't know about the curve with oid %s."
                            "I only know about these: %s" %
                            (oid_curve, [c.name for c in curves]))



"""
Common functions for providing cross-python version compatibility.
"""
import sys
from six import integer_types


def str_idx_as_int(string, index):
    """Take index'th byte from string, return as integer"""
    val = string[index]
    if isinstance(val, integer_types):
        return val
    return ord(val)


if sys.version_info < (3, 0):
    def normalise_bytes(buffer_object):
        """Cast the input into array of bytes."""
        # flake8 runs on py3 where `buffer` indeed doesn't exist...
        return buffer(buffer_object)  # noqa: F821

    def hmac_compat(ret):
        return ret

else:
    if sys.version_info < (3, 4):
        # on python 3.3 hmac.hmac.update() accepts only bytes, on newer
        # versions it does accept memoryview() also
        def hmac_compat(data):
            if not isinstance(data, bytes):
                return bytes(data)
            return data
    else:
        def hmac_compat(data):
            return data

    def normalise_bytes(buffer_object):
        """Cast the input into array of bytes."""
        return memoryview(buffer_object).cast('B')



#!/usr/bin/env python

import io
import os

from setuptools import setup
import versioneer

commands = versioneer.get_cmdclass().copy()

# Use README.md to set markdown long_description
directory = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(directory, "README.md")
with io.open(readme_path, encoding="utf-8") as read_file:
    long_description = read_file.read()

setup(name="ecdsa",
      version=versioneer.get_version(),
      description="ECDSA cryptographic signature library (pure python)",
      long_description=long_description,
      long_description_content_type='text/markdown',
      author="Brian Warner",
      author_email="warner@lothar.com",
      url="http://github.com/warner/python-ecdsa",
      packages=["ecdsa"],
      package_dir={"": "src"},
      license="MIT",
      cmdclass=commands,
      python_requires=">=2.6, !=3.0.*, !=3.1.*, !=3.2.*",
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8",
      ],
      install_requires=['six>=1.9.0'],
      extras_require={
          'gmpy2': 'gmpy2',
          'gmpy': 'gmpy',
      },
      )



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Implementation of elliptic curves, for cryptographic applications.
#
# This module doesn't provide any way to choose a random elliptic
# curve, nor to verify that an elliptic curve was chosen randomly,
# because one can simply use NIST's standard curves.
#
# Notes from X9.62-1998 (draft):
#   Nomenclature:
#     - Q is a public key.
#     The "Elliptic Curve Domain Parameters" include:
#     - q is the "field size", which in our case equals p.
#     - p is a big prime.
#     - G is a point of prime order (5.1.1.1).
#     - n is the order of G (5.1.1.1).
#   Public-key validation (5.2.2):
#     - Verify that Q is not the point at infinity.
#     - Verify that X_Q and Y_Q are in [0,p-1].
#     - Verify that Q is on the curve.
#     - Verify that nQ is the point at infinity.
#   Signature generation (5.3):
#     - Pick random k from [1,n-1].
#   Signature checking (5.4.2):
#     - Verify that r and s are in [1,n-1].
#
# Version of 2008.11.25.
#
# Revision history:
#    2005.12.31 - Initial version.
#    2008.11.25 - Change CurveFp.is_on to contains_point.
#
# Written in 2005 by Peter Pearson and placed in the public domain.

from __future__ import division

try:
    from gmpy2 import mpz
    GMPY = True
except ImportError:
    try:
        from gmpy import mpz
        GMPY = True
    except ImportError:
        GMPY = False


from six import python_2_unicode_compatible
from . import numbertheory
from ._rwlock import RWLock


@python_2_unicode_compatible
class CurveFp(object):
  """Elliptic Curve over the field of integers modulo a prime."""

  if GMPY:
      def __init__(self, p, a, b, h=None):
        """
        The curve of points satisfying y^2 = x^3 + a*x + b (mod p).

        h is an integer that is the cofactor of the elliptic curve domain
        parameters; it is the number of points satisfying the elliptic curve
        equation divided by the order of the base point. It is used for selection
        of efficient algorithm for public point verification.
        """
        self.__p = mpz(p)
        self.__a = mpz(a)
        self.__b = mpz(b)
        # h is not used in calculations and it can be None, so don't use
        # gmpy with it
        self.__h = h
  else:
      def __init__(self, p, a, b, h=None):
        """
        The curve of points satisfying y^2 = x^3 + a*x + b (mod p).

        h is an integer that is the cofactor of the elliptic curve domain
        parameters; it is the number of points satisfying the elliptic curve
        equation divided by the order of the base point. It is used for selection
        of efficient algorithm for public point verification.
        """
        self.__p = p
        self.__a = a
        self.__b = b
        self.__h = h

  def __eq__(self, other):
    if isinstance(other, CurveFp):
      """Return True if the curves are identical, False otherwise."""
      return self.__p == other.__p \
          and self.__a == other.__a \
          and self.__b == other.__b
    return NotImplemented

  def __hash__(self):
    return hash((self.__p, self.__a, self.__b))

  def p(self):
    return self.__p

  def a(self):
    return self.__a

  def b(self):
    return self.__b

  def cofactor(self):
    return self.__h

  def contains_point(self, x, y):
    """Is the point (x,y) on this curve?"""
    return (y * y - ((x * x + self.__a) * x + self.__b)) % self.__p == 0

  def __str__(self):
    return "CurveFp(p=%d, a=%d, b=%d, h=%d)" % (
        self.__p, self.__a, self.__b, self.__h)


class PointJacobi(object):
  """
  Point on an elliptic curve. Uses Jacobi coordinates.

  In Jacobian coordinates, there are three parameters, X, Y and Z.
  They correspond to affine parameters 'x' and 'y' like so:

  x = X / Z²
  y = Y / Z³
  """
  def __init__(self, curve, x, y, z, order=None, generator=False):
      """
      Initialise a point that uses Jacobi representation internally.

      :param CurveFp curve: curve on which the point resides
      :param int x: the X parameter of Jacobi representation (equal to x when
        converting from affine coordinates
      :param int y: the Y parameter of Jacobi representation (equal to y when
        converting from affine coordinates
      :param int z: the Z parameter of Jacobi representation (equal to 1 when
        converting from affine coordinates
      :param int order: the point order, must be non zero when using
        generator=True
      :param bool generator: the point provided is a curve generator, as
        such, it will be commonly used with scalar multiplication. This will
        cause to precompute multiplication table for it
      """
      self.__curve = curve
      # since it's generally better (faster) to use scaled points vs unscaled
      # ones, use writer-biased RWLock for locking:
      self._scale_lock = RWLock()
      if GMPY:
          self.__x = mpz(x)
          self.__y = mpz(y)
          self.__z = mpz(z)
          self.__order = order and mpz(order)
      else:
          self.__x = x
          self.__y = y
          self.__z = z
          self.__order = order
      self.__precompute = []
      if generator:
          assert order
          i = 1
          order *= 2
          doubler = PointJacobi(curve, x, y, z, order)
          order *= 2
          self.__precompute.append((doubler.x(), doubler.y()))

          while i < order:
              i *= 2
              doubler = doubler.double().scale()
              self.__precompute.append((doubler.x(), doubler.y()))

  def __eq__(self, other):
      """Compare two points with each-other."""
      try:
          self._scale_lock.reader_acquire()
          if other is INFINITY:
              return not self.__y or not self.__z
          x1, y1, z1 = self.__x, self.__y, self.__z
      finally:
          self._scale_lock.reader_release()
      if isinstance(other, Point):
          x2, y2, z2 = other.x(), other.y(), 1
      elif isinstance(other, PointJacobi):
          try:
              other._scale_lock.reader_acquire()
              x2, y2, z2 = other.__x, other.__y, other.__z
          finally:
              other._scale_lock.reader_release()
      else:
          return NotImplemented
      if self.__curve != other.curve():
          return False
      p = self.__curve.p()

      zz1 = z1 * z1 % p
      zz2 = z2 * z2 % p

      # compare the fractions by bringing them to the same denominator
      # depend on short-circuit to save 4 multiplications in case of inequality
      return (x1 * zz2 - x2 * zz1) % p == 0 and \
             (y1 * zz2 * z2 - y2 * zz1 * z1) % p == 0

  def order(self):
      """Return the order of the point.

      None if it is undefined.
      """
      return self.__order

  def curve(self):
      """Return curve over which the point is defined."""
      return self.__curve

  def x(self):
      """
      Return affine x coordinate.

      This method should be used only when the 'y' coordinate is not needed.
      It's computationally more efficient to use `to_affine()` and then
      call x() and y() on the returned instance. Or call `scale()`
      and then x() and y() on the returned instance.
      """
      try:
          self._scale_lock.reader_acquire()
          if self.__z == 1:
              return self.__x
          x = self.__x
          z = self.__z
      finally:
          self._scale_lock.reader_release()
      p = self.__curve.p()
      z = numbertheory.inverse_mod(z, p)
      return x * z**2 % p

  def y(self):
      """
      Return affine y coordinate.

      This method should be used only when the 'x' coordinate is not needed.
      It's computationally more efficient to use `to_affine()` and then
      call x() and y() on the returned instance. Or call `scale()`
      and then x() and y() on the returned instance.
      """
      try:
          self._scale_lock.reader_acquire()
          if self.__z == 1:
              return self.__y
          y = self.__y
          z = self.__z
      finally:
          self._scale_lock.reader_release()
      p = self.__curve.p()
      z = numbertheory.inverse_mod(z, p)
      return y * z**3 % p

  def scale(self):
      """
      Return point scaled so that z == 1.

      Modifies point in place, returns self.
      """
      try:
          self._scale_lock.reader_acquire()
          if self.__z == 1:
              return self
      finally:
          self._scale_lock.reader_release()

      try:
          self._scale_lock.writer_acquire()
          # scaling already scaled point is safe (as inverse of 1 is 1) and
          # quick so we don't need to optimise for the unlikely event when
          # two threads hit the lock at the same time
          p = self.__curve.p()
          z_inv = numbertheory.inverse_mod(self.__z, p)
          zz_inv = z_inv * z_inv % p
          self.__x = self.__x * zz_inv % p
          self.__y = self.__y * zz_inv * z_inv % p
          # we are setting the z last so that the check above will return true
          # only after all values were already updated
          self.__z = 1
      finally:
          self._scale_lock.writer_release()
      return self

  def to_affine(self):
      """Return point in affine form."""
      if not self.__y or not self.__z:
          return INFINITY
      self.scale()
      # after point is scaled, it's immutable, so no need to perform locking
      return Point(self.__curve, self.__x,
                   self.__y, self.__order)

  @staticmethod
  def from_affine(point, generator=False):
      """Create from an affine point.

      :param bool generator: set to True to make the point to precalculate
        multiplication table - useful for public point when verifying many
        signatures (around 100 or so) or for generator points of a curve.
      """
      return PointJacobi(point.curve(), point.x(), point.y(), 1,
                         point.order(), generator)

  # plese note that all the methods that use the equations from hyperelliptic
  # are formatted in a way to maximise performance.
  # Things that make code faster: multiplying instead of taking to the power
  # (`xx = x * x; xxxx = xx * xx % p` is faster than `xxxx = x**4 % p` and
  # `pow(x, 4, p)`),
  # multiple assignments at the same time (`x1, x2 = self.x1, self.x2` is
  # faster than `x1 = self.x1; x2 = self.x2`),
  # similarly, sometimes the `% p` is skipped if it makes the calculation
  # faster and the result of calculation is later reduced modulo `p`

  def _double_with_z_1(self, X1, Y1, p, a):
      """Add a point to itself with z == 1."""
      # after:
      # http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#doubling-mdbl-2007-bl
      XX, YY = X1 * X1 % p, Y1 * Y1 % p
      if not YY:
          return 0, 0, 1
      YYYY = YY * YY % p
      S = 2 * ((X1 + YY)**2 - XX - YYYY) % p
      M = 3 * XX + a
      T = (M * M - 2 * S) % p
      # X3 = T
      Y3 = (M * (S - T) - 8 * YYYY) % p
      Z3 = 2 * Y1 % p
      return T, Y3, Z3

  def _double(self, X1, Y1, Z1, p, a):
      """Add a point to itself, arbitrary z."""
      if Z1 == 1:
          return self._double_with_z_1(X1, Y1, p, a)
      if not Z1:
          return 0, 0, 1
      # after:
      # http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#doubling-dbl-2007-bl
      XX, YY = X1 * X1 % p, Y1 * Y1 % p
      if not YY:
          return 0, 0, 1
      YYYY = YY * YY % p
      ZZ = Z1 * Z1 % p
      S = 2 * ((X1 + YY)**2 - XX - YYYY) % p
      M = (3 * XX + a * ZZ * ZZ) % p
      T = (M * M - 2 * S) % p
      # X3 = T
      Y3 = (M * (S - T) - 8 * YYYY) % p
      Z3 = ((Y1 + Z1)**2 - YY - ZZ) % p

      return T, Y3, Z3

  def double(self):
      """Add a point to itself."""
      if not self.__y:
          return INFINITY

      p, a = self.__curve.p(), self.__curve.a()

      try:
          self._scale_lock.reader_acquire()
          X1, Y1, Z1 = self.__x, self.__y, self.__z
      finally:
          self._scale_lock.reader_release()

      X3, Y3, Z3 = self._double(X1, Y1, Z1, p, a)

      if not Y3 or not Z3:
          return INFINITY
      return PointJacobi(self.__curve, X3, Y3, Z3, self.__order)

  def _add_with_z_1(self, X1, Y1, X2, Y2, p):
      """add points when both Z1 and Z2 equal 1"""
      # after:
      # http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#addition-mmadd-2007-bl
      H = X2 - X1
      HH = H * H
      I = 4 * HH % p
      J = H * I
      r = 2 * (Y2 - Y1)
      if not H and not r:
          return self._double_with_z_1(X1, Y1, p, self.__curve.a())
      V = X1 * I
      X3 = (r**2 - J - 2 * V) % p
      Y3 = (r * (V - X3) - 2 * Y1 * J) % p
      Z3 = 2 * H % p
      return X3, Y3, Z3

  def _add_with_z_eq(self, X1, Y1, Z1, X2, Y2, p):
      """add points when Z1 == Z2"""
      # after:
      # http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#addition-zadd-2007-m
      A = (X2 - X1)**2 % p
      B = X1 * A % p
      C = X2 * A
      D = (Y2 - Y1)**2 % p
      if not A and not D:
          return self._double(X1, Y1, Z1, p, self.__curve.a())
      X3 = (D - B - C) % p
      Y3 = ((Y2 - Y1) * (B - X3) - Y1 * (C - B)) % p
      Z3 = Z1 * (X2 - X1) % p
      return X3, Y3, Z3

  def _add_with_z2_1(self, X1, Y1, Z1, X2, Y2, p):
      """add points when Z2 == 1"""
      # after:
      # http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#addition-madd-2007-bl
      Z1Z1 = Z1 * Z1 % p
      U2, S2 = X2 * Z1Z1 % p, Y2 * Z1 * Z1Z1 % p
      H = (U2 - X1) % p
      HH = H * H % p
      I = 4 * HH % p
      J = H * I
      r = 2 * (S2 - Y1) % p
      if not r and not H:
          return self._double_with_z_1(X2, Y2, p, self.__curve.a())
      V = X1 * I
      X3 = (r * r - J - 2 * V) % p
      Y3 = (r * (V - X3) - 2 * Y1 * J) % p
      Z3 = ((Z1 + H)**2 - Z1Z1 - HH) % p
      return X3, Y3, Z3

  def _add_with_z_ne(self, X1, Y1, Z1, X2, Y2, Z2, p):
      """add points with arbitrary z"""
      # after:
      # http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#addition-add-2007-bl
      Z1Z1 = Z1 * Z1 % p
      Z2Z2 = Z2 * Z2 % p
      U1 = X1 * Z2Z2 % p
      U2 = X2 * Z1Z1 % p
      S1 = Y1 * Z2 * Z2Z2 % p
      S2 = Y2 * Z1 * Z1Z1 % p
      H = U2 - U1
      I = 4 * H * H % p
      J = H * I % p
      r = 2 * (S2 - S1) % p
      if not H and not r:
          return self._double(X1, Y1, Z1, p, self.__curve.a())
      V = U1 * I
      X3 = (r * r - J - 2 * V) % p
      Y3 = (r * (V - X3) - 2 * S1 * J) % p
      Z3 = ((Z1 + Z2)**2 - Z1Z1 - Z2Z2) * H % p

      return X3, Y3, Z3

  def __radd__(self, other):
      """Add other to self."""
      return self + other

  def _add(self, X1, Y1, Z1, X2, Y2, Z2, p):
      """add two points, select fastest method."""
      if not Y1 or not Z1:
          return X2, Y2, Z2
      if not Y2 or not Z2:
          return X1, Y1, Z1
      if Z1 == Z2:
          if Z1 == 1:
              return self._add_with_z_1(X1, Y1, X2, Y2, p)
          return self._add_with_z_eq(X1, Y1, Z1, X2, Y2, p)
      if Z1 == 1:
          return self._add_with_z2_1(X2, Y2, Z2, X1, Y1, p)
      if Z2 == 1:
          return self._add_with_z2_1(X1, Y1, Z1, X2, Y2, p)
      return self._add_with_z_ne(X1, Y1, Z1, X2, Y2, Z2, p)

  def __add__(self, other):
      """Add two points on elliptic curve."""
      if self == INFINITY:
          return other
      if other == INFINITY:
          return self
      if isinstance(other, Point):
          other = PointJacobi.from_affine(other)
      if self.__curve != other.__curve:
          raise ValueError("The other point is on different curve")

      p = self.__curve.p()
      try:
          self._scale_lock.reader_acquire()
          X1, Y1, Z1 = self.__x, self.__y, self.__z
      finally:
          self._scale_lock.reader_release()
      try:
          other._scale_lock.reader_acquire()
          X2, Y2, Z2 = other.__x, other.__y, other.__z
      finally:
          other._scale_lock.reader_release()
      X3, Y3, Z3 = self._add(X1, Y1, Z1, X2, Y2, Z2, p)

      if not Y3 or not Z3:
          return INFINITY
      return PointJacobi(self.__curve, X3, Y3, Z3, self.__order)

  def __rmul__(self, other):
      """Multiply point by an integer."""
      return self * other

  def _mul_precompute(self, other):
      """Multiply point by integer with precomputation table."""
      X3, Y3, Z3, p = 0, 0, 1, self.__curve.p()
      _add = self._add
      for X2, Y2 in self.__precompute:
          if other % 2:
              if other % 4 >= 2:
                  other = (other + 1)//2
                  X3, Y3, Z3 = _add(X3, Y3, Z3, X2, -Y2, 1, p)
              else:
                  other = (other - 1)//2
                  X3, Y3, Z3 = _add(X3, Y3, Z3, X2, Y2, 1, p)
          else:
              other //= 2

      if not Y3 or not Z3:
          return INFINITY
      return PointJacobi(self.__curve, X3, Y3, Z3, self.__order)

  @staticmethod
  def _naf(mult):
      """Calculate non-adjacent form of number."""
      ret = []
      while mult:
          if mult % 2:
              nd = mult % 4
              if nd >= 2:
                  nd = nd - 4
              ret += [nd]
              mult -= nd
          else:
              ret += [0]
          mult //= 2
      return ret

  def __mul__(self, other):
      """Multiply point by an integer."""
      if not self.__y or not other:
          return INFINITY
      if other == 1:
          return self
      if self.__order:
          # order*2 as a protection for Minerva
          other = other % (self.__order*2)
      if self.__precompute:
          return self._mul_precompute(other)

      self = self.scale()
      # once scaled, point is immutable, not need to lock
      X2, Y2 = self.__x, self.__y
      X3, Y3, Z3 = 0, 0, 1
      p, a = self.__curve.p(), self.__curve.a()
      _double = self._double
      _add = self._add
      # since adding points when at least one of them is scaled
      # is quicker, reverse the NAF order
      for i in reversed(self._naf(other)):
          X3, Y3, Z3 = _double(X3, Y3, Z3, p, a)
          if i < 0:
              X3, Y3, Z3 = _add(X3, Y3, Z3, X2, -Y2, 1, p)
          elif i > 0:
              X3, Y3, Z3 = _add(X3, Y3, Z3, X2, Y2, 1, p)

      if not Y3 or not Z3:
          return INFINITY

      return PointJacobi(self.__curve, X3, Y3, Z3, self.__order)

  @staticmethod
  def _leftmost_bit(x):
      """Return integer with the same magnitude as x but hamming weight of 1"""
      assert x > 0
      result = 1
      while result <= x:
        result = 2 * result
      return result // 2

  def mul_add(self, self_mul, other, other_mul):
      """
      Do two multiplications at the same time, add results.

      calculates self*self_mul + other*other_mul
      """
      if other is INFINITY or other_mul == 0:
          return self * self_mul
      if self_mul == 0:
          return other * other_mul
      if not isinstance(other, PointJacobi):
          other = PointJacobi.from_affine(other)
      # when the points have precomputed answers, then multiplying them alone
      # is faster (as it uses NAF)
      if self.__precompute and other.__precompute:
          return self * self_mul + other * other_mul

      if self.__order:
          self_mul = self_mul % self.__order
          other_mul = other_mul % self.__order

      i = self._leftmost_bit(max(self_mul, other_mul))*2
      X3, Y3, Z3 = 0, 0, 1
      p, a = self.__curve.p(), self.__curve.a()
      self = self.scale()
      # after scaling, point is immutable, no need for locking
      X1, Y1 = self.__x, self.__y
      other = other.scale()
      X2, Y2 = other.__x, other.__y
      both = (self + other).scale()
      X4, Y4 = both.__x, both.__y
      _double = self._double
      _add = self._add
      while i > 1:
          X3, Y3, Z3 = _double(X3, Y3, Z3, p, a)
          i = i // 2

          if self_mul & i and other_mul & i:
              X3, Y3, Z3 = _add(X3, Y3, Z3, X4, Y4, 1, p)
          elif self_mul & i:
              X3, Y3, Z3 = _add(X3, Y3, Z3, X1, Y1, 1, p)
          elif other_mul & i:
              X3, Y3, Z3 = _add(X3, Y3, Z3, X2, Y2, 1, p)

      if not Y3 or not Z3:
          return INFINITY

      return PointJacobi(self.__curve, X3, Y3, Z3, self.__order)

  def __neg__(self):
      """Return negated point."""
      try:
          self._scale_lock.reader_acquire()
          return PointJacobi(self.__curve, self.__x, -self.__y, self.__z,
                             self.__order)
      finally:
          self._scale_lock.reader_release()


class Point(object):
  """A point on an elliptic curve. Altering x and y is forbidding,
     but they can be read by the x() and y() methods."""
  def __init__(self, curve, x, y, order=None):
    """curve, x, y, order; order (optional) is the order of this point."""
    self.__curve = curve
    if GMPY:
        self.__x = x and mpz(x)
        self.__y = y and mpz(y)
        self.__order = order and mpz(order)
    else:
        self.__x = x
        self.__y = y
        self.__order = order
    # self.curve is allowed to be None only for INFINITY:
    if self.__curve:
      assert self.__curve.contains_point(x, y)
    # for curves with cofactor 1, all points that are on the curve are scalar
    # multiples of the base point, so performing multiplication is not
    # necessary to verify that. See Section 3.2.2.1 of SEC 1 v2
    if curve and curve.cofactor() != 1 and order:
      assert self * order == INFINITY

  def __eq__(self, other):
    """Return True if the points are identical, False otherwise."""
    if isinstance(other, Point):
      return self.__curve == other.__curve \
          and self.__x == other.__x \
          and self.__y == other.__y
    return NotImplemented

  def __neg__(self):
    return Point(self.__curve, self.__x, self.__curve.p() - self.__y)

  def __add__(self, other):
    """Add one point to another point."""

    # X9.62 B.3:

    if not isinstance(other, Point):
        return NotImplemented
    if other == INFINITY:
      return self
    if self == INFINITY:
      return other
    assert self.__curve == other.__curve
    if self.__x == other.__x:
      if (self.__y + other.__y) % self.__curve.p() == 0:
        return INFINITY
      else:
        return self.double()

    p = self.__curve.p()

    l = ((other.__y - self.__y) * \
         numbertheory.inverse_mod(other.__x - self.__x, p)) % p

    x3 = (l * l - self.__x - other.__x) % p
    y3 = (l * (self.__x - x3) - self.__y) % p

    return Point(self.__curve, x3, y3)

  def __mul__(self, other):
    """Multiply a point by an integer."""

    def leftmost_bit(x):
      assert x > 0
      result = 1
      while result <= x:
        result = 2 * result
      return result // 2

    e = other
    if e == 0 or (self.__order and e % self.__order == 0):
      return INFINITY
    if self == INFINITY:
      return INFINITY
    if e < 0:
      return (-self) * (-e)

    # From X9.62 D.3.2:

    e3 = 3 * e
    negative_self = Point(self.__curve, self.__x, -self.__y, self.__order)
    i = leftmost_bit(e3) // 2
    result = self
    # print_("Multiplying %s by %d (e3 = %d):" % (self, other, e3))
    while i > 1:
      result = result.double()
      if (e3 & i) != 0 and (e & i) == 0:
        result = result + self
      if (e3 & i) == 0 and (e & i) != 0:
        result = result + negative_self
      # print_(". . . i = %d, result = %s" % ( i, result ))
      i = i // 2

    return result

  def __rmul__(self, other):
    """Multiply a point by an integer."""

    return self * other

  def __str__(self):
    if self == INFINITY:
      return "infinity"
    return "(%d,%d)" % (self.__x, self.__y)

  def double(self):
    """Return a new point that is twice the old."""

    if self == INFINITY:
      return INFINITY

    # X9.62 B.3:

    p = self.__curve.p()
    a = self.__curve.a()

    l = ((3 * self.__x * self.__x + a) * \
         numbertheory.inverse_mod(2 * self.__y, p)) % p

    x3 = (l * l - 2 * self.__x) % p
    y3 = (l * (self.__x - x3) - self.__y) % p

    return Point(self.__curve, x3, y3)

  def x(self):
    return self.__x

  def y(self):
    return self.__y

  def curve(self):
    return self.__curve

  def order(self):
    return self.__order


# This one point is the Point At Infinity for all purposes:
INFINITY = Point(None, None, None)
